 #
# This file is part of pySMT.
#
#   Copyright 2014 Andrea Micheli and Marco Gario
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
import re
from collections import namedtuple

import pysmt.typing as types
from pysmt.environment import get_env
from pysmt.exceptions import PysmtSyntaxError, UndefinedSymbolError
from pysmt.constants import Fraction


def HRParser(env=None):
    """Parser for HR format of pySMT."""
    return PrattParser(HRLexer, env=env)


def parse(string):
    """Parse an hr-string."""
    return HRParser().parse(string)


# Rules for the Lexer
Rule = namedtuple('Rule', ['regex', 'symbol', 'is_functional'])


class Lexer(object):
    """This class produces a stream of token objects for the Pratt-parser.

    The rules for the token are store in self.rules. Call self.compile() after
    modifying rules.
    """

    def __init__(self, env=None):
        if env is None:
            env = get_env()
        self.env = env
        self.mgr = env.formula_manager
        self.get_type = env.stc.get_type

        self.rules = []
        self.scanner = None
        self.eoi = EndOfInput()

    def compile(self):
        self.scanner = re.compile("|".join(rule.regex for rule in self.rules),
                                  re.DOTALL | re.VERBOSE)
    def lexing_error(self, read):
        raise PysmtSyntaxError("Unexpected input: %s" % read)

    def tokenize(self, data):
        """The token generator for the given string"""
        for match in re.finditer(self.scanner, data):
            for idx, capture in enumerate(match.groups()):
                if capture is not None:
                    rule = self.rules[idx]
                    symbol = rule.symbol
                    if rule.symbol is not None:
                        if rule.is_functional:
                            yield symbol(capture)
                        else:
                            yield symbol
                    break
        yield self.eoi

# EOC Lexer


class GrammarSymbol(object):
    """Base class for all the parsing tokens"""

    def __init__(self, lbp=0, value=None, payload=None):
        self.lbp = lbp
        self.value = value
        self.payload = payload

    def nud(self, parser):
        raise PysmtSyntaxError("Syntax error at token '%s'." % parser.token)

    def led(self, parser, left):
        raise PysmtSyntaxError("Syntax error at token '%s' (Read: '%s')." % \
                          (parser.token, left))

#
# Precedence table (from low priority to high priority):
#
# 10 : Xor, Implies, Iff
# 20 : Forall, Exists
# 30 : Or
# 40 : And
# 50 : Not
# 60 : LE, LT, Equals, GE, GT, BVULE, BVUGE, BVULT, BVUGT, BVSLE, BVSGE, BVSGT
# 70 : Plus, Minus, BVAdd, BVSub
# 80 : Times, Div, BVMul, BVUDiv, BVSDiv, BVSRem, BVURem
# 90 : BVLShl, BVLShr, BVAShr, BVConcat, BVXor, BVRor, BVRol, BVZExt, BVSExt, BVComp
# 100 : ToReal Uminus BVNeg BVToNat
# 200 : ()
# 300 : []

class HRLexer(Lexer):
    """Produces a stream of token objects for the Human-Readable format."""

    def __init__(self, env=None):
        Lexer.__init__(self, env=env)

        hr_rules = [
            Rule(r"(\s+)", None, False),                    # whitespace
            Rule(r"(-?\d+/\d+)", self.real_constant, True), # fractions
            Rule(r"(-?\d+\.\d+)", self.real_constant, True),# decimals
            Rule(r"(-?\d+_\d+)", self.bv_constant, True),# bv
            Rule(r"(-?\d+)", self.int_constant, True),# integer literals
            Rule(r"\"(.*?)\"", self.string_constant, True), # String Constant
            Rule(r"BV\{(\d+)\}", self.bv_type, True),# BV Type
            Rule(r"(Array\{)", OpenArrayTypeTok(), False),# Array Type
            Rule(r"(Int)", IntTypeTok(), False),# Int Type
            Rule(r"(Real)", RealTypeTok(), False),# Real Type
            Rule(r"(Bool)", BoolTypeTok(), False),# Bool Type
            Rule(r"(&)", InfixOpAdapter(self.AndOrBVAnd, 40), False),# conjunction
            Rule(r"(\|)", InfixOpAdapter(self.OrOrBVOr, 30), False),# disjunction
            Rule(r"(!)", UnaryOpAdapter(self.NotOrBVNot, 50), False),# negation
            Rule(r"(\()", OpenPar(), False),# open parenthesis
            Rule(r"(\))", ClosePar(), False),# closed parenthesis
            Rule(r"(\[)", OpenBrak(), False),# open parenthesis
            Rule(r"(\])", CloseBrak(), False),# closed parenthesis
            Rule(r"(\})", CloseBrace(), False),# closed parenthesis
            Rule(r"(<<)", InfixOpAdapter(self.mgr.BVLShl, 90), False),# Shl
            Rule(r"(>>)", InfixOpAdapter(self.mgr.BVLShr, 90), False),# Shr
            Rule(r"(a>>)", InfixOpAdapter(self.mgr.BVAShr, 90), False),# AShr
            Rule(r"(<->)", InfixOpAdapter(self.mgr.Iff, 10), False),# iff
            Rule(r"(->)", InfixOpAdapter(self.mgr.Implies, 10), False),# implies
            Rule(r"(u<=)", InfixOpAdapter(self.mgr.BVULE, 60), False),# bvule
            Rule(r"(u>=)", InfixOpAdapter(self.mgr.BVUGE, 60), False),# bvuge
            Rule(r"(u<)", InfixOpAdapter(self.mgr.BVULT, 60), False),# bvult
            Rule(r"(u>)", InfixOpAdapter(self.mgr.BVUGT, 60), False),# bvugt
            Rule(r"(s<=)", InfixOpAdapter(self.mgr.BVSLE, 60), False),# bvsle
            Rule(r"(s>=)", InfixOpAdapter(self.mgr.BVSGE, 60), False),# bvsge
            Rule(r"(s<)", InfixOpAdapter(self.mgr.BVSLT, 60), False),# bvslt
            Rule(r"(s>)", InfixOpAdapter(self.mgr.BVSGT, 60), False),# bvsgt
            Rule(r"(>=)", InfixOpAdapter(self.mgr.GE, 60), False),# ge
            Rule(r"(<=)", InfixOpAdapter(self.mgr.LE, 60), False),# le
            Rule(r"(>)", InfixOpAdapter(self.mgr.GT, 60), False),# gt
            Rule(r"(<)", InfixOpAdapter(self.mgr.LT, 60), False),# lt
            Rule(r"(=)", InfixOpAdapter(self.mgr.Equals, 60), False),# eq
            Rule(r"(\+)", InfixOpAdapter(self.PlusOrBVAdd, 70), False),# plus
            Rule(r"(-)", InfixOrUnaryOpAdapter(self.MinusOrBVSub, self.UMinusOrBvNeg, 70, 100), False),# minus
            Rule(r"(\*)", InfixOpAdapter(self.TimesOrBVMul, 80), False),# times
            Rule(r"(\^)", InfixOpAdapter(self.mgr.Pow, 80), False),# pow
            Rule(r"(u/)", InfixOpAdapter(self.mgr.BVUDiv, 80), False),# udiv
            Rule(r"(s/)", InfixOpAdapter(self.mgr.BVSDiv, 80), False),# sdiv
            Rule(r"(/)", InfixOpAdapter(self.mgr.Div, 80), False),# div
            Rule(r"(s%)", InfixOpAdapter(self.mgr.BVSRem, 80), False),# srem
            Rule(r"(u%)", InfixOpAdapter(self.mgr.BVURem, 80), False),# urem
            Rule(r"(\?)", ExprIf(), False), # question
            Rule(r"(:=)", ArrStore(), False),# ArrStore
            Rule(r"(::)", InfixOpAdapter(self.mgr.BVConcat, 90), False),# BVXor
            Rule(r"(:)", ExprElse(), False),# colon
            Rule(r"(False)", Constant(self.mgr.FALSE()), False), # False
            Rule(r"(True)", Constant(self.mgr.TRUE()), False),# True
            Rule(r"(,)", ExprComma(), False),# comma
            Rule(r"(\.)", ExprDot(), False),# dot
            Rule(r"(xor)", InfixOpAdapter(self.mgr.BVXor, 10), False),# BVXor
            Rule(r"(ROR)", InfixOpAdapter(self.BVHack(self.mgr.BVRor), 90), False),# BVRor
            Rule(r"(ROL)", InfixOpAdapter(self.BVHack(self.mgr.BVRol), 90), False),# BVRol
            Rule(r"(ZEXT)", InfixOpAdapter(self.BVHack(self.mgr.BVZExt), 90), False),# BVZext
            Rule(r"(SEXT)", InfixOpAdapter(self.BVHack(self.mgr.BVSExt), 90), False),# BVSext
            Rule(r"(bvcomp)", InfixOpAdapter(self.mgr.BVComp, 90), False),#
            Rule(r"(forall)", Quantifier(self.mgr.ForAll, 20), False),#
            Rule(r"(exists)", Quantifier(self.mgr.Exists, 20), False),#
            Rule(r"(ToReal)", UnaryOpAdapter(self.mgr.ToReal, 100), False),#
            Rule(r"(str\.len)", FunctionCallAdapter(self.mgr.StrLength, 100), False), # str_length
            Rule(r"(str\.\+\+)", FunctionCallAdapter(self.mgr.StrConcat, 100), False), # str_concat
            Rule(r"(str\.at)", FunctionCallAdapter(self.mgr.StrCharAt, 100), False), # str_charat
            Rule(r"(str\.contains)", FunctionCallAdapter(self.mgr.StrContains, 100), False), # str_contains
            Rule(r"(str\.indexof)", FunctionCallAdapter(self.mgr.StrIndexOf, 100), False), # str_indexof
            Rule(r"(str\.replace)", FunctionCallAdapter(self.mgr.StrReplace, 100), False), # str_replace
            Rule(r"(str\.substr)", FunctionCallAdapter(self.mgr.StrSubstr, 100), False), # str_substr
            Rule(r"(str\.prefixof)", FunctionCallAdapter(self.mgr.StrPrefixOf, 100), False), # str_prefixof
            Rule(r"(str\.suffixof)", FunctionCallAdapter(self.mgr.StrSuffixOf, 100), False), # str_suffixof
            Rule(r"(str\.to\.int)", FunctionCallAdapter(self.mgr.StrToInt, 100), False), # str_to_int
            Rule(r"(int\.to\.str)", FunctionCallAdapter(self.mgr.IntToStr, 100), False), # int_to_str
            Rule(r"(bv2nat)", UnaryOpAdapter(self.mgr.BVToNatural, 100), False),#
            Rule(r"'(.*?)'", self.identifier, True), # quoted identifiers
            Rule(r"([A-Za-z_][A-Za-z0-9_]*)", self.identifier, True),# identifiers
            Rule(r"(.)", self.lexing_error, True), # input error
        ]

        self.rules += hr_rules

        self.compile()

    def bv_type(self, read):
        return BVTypeTok(int(read))

    def real_constant(self, read):
        return Constant(self.mgr.Real(Fraction(read)))

    def bv_constant(self, read):
        v, w = read.split("_")
        return Constant(self.mgr.BV(int(v), width=int(w)))

    def int_constant(self, read):
        return Constant(self.mgr.Int(int(read)))

    def string_constant(self, read):
        return Constant(self.mgr.String(read))

    def identifier(self, read):
        return Identifier(read, env=self.env)

    def UMinusOrBvNeg(self, x):
        ty = self.get_type(x)
        if ty.is_int_type():
            return self.mgr.Times(self.mgr.Int(-1), x)
        elif ty.is_real_type():
            return self.mgr.Times(self.mgr.Real(-1), x)
        else:
            return self.mgr.BVNeg(x)

    def AndOrBVAnd(self, l, r):
        if self.get_type(l).is_bool_type():
            return self.mgr.And(l, r)
        else:
            return self.mgr.BVAnd(l, r)

    def OrOrBVOr(self, l, r):
        if self.get_type(l).is_bool_type():
            return self.mgr.Or(l, r)
        else:
            return self.mgr.BVOr(l, r)

    def NotOrBVNot(self, x):
        if self.get_type(x).is_bool_type():
            return self.mgr.Not(x)
        else:
            return self.mgr.BVNot(x)

    def PlusOrBVAdd(self, l, r):
        if self.get_type(l).is_bv_type():
            return self.mgr.BVAdd(l, r)
        else:
            return self.mgr.Plus(l, r)

    def MinusOrBVSub(self, l, r):
        if self.get_type(l).is_bv_type():
            return self.mgr.BVSub(l, r)
        else:
            return self.mgr.Minus(l, r)

    def TimesOrBVMul(self, l, r):
        if self.get_type(l).is_bv_type():
            return self.mgr.BVMul(l, r)
        else:
            return self.mgr.Times(l, r)

    def BVHack(self, op):
        def _res(a, b):
            if b.is_constant():
                return op(a, b.constant_value())
            else:
                raise PysmtSyntaxError("Constant expected, got '%s'" % b)
        return _res


# EOC HRLexer

#
# Grammar tokens
#

# Placeholder tokens


class CloseBrace(GrammarSymbol):
    pass

class ArrStore(GrammarSymbol):
    pass


class BVTypeTok(GrammarSymbol):
    def __init__(self, width):
        GrammarSymbol.__init__(self)
        self.width = width

    def nud(self, parser):
        return types.BVType(self.width)

class IntTypeTok(GrammarSymbol):
    def nud(self, parser):
        return types.INT

class RealTypeTok(GrammarSymbol):
    def nud(self, parser):
        return types.REAL

class BoolTypeTok(GrammarSymbol):
    def nud(self, parser):
        return types.BOOL

class Constant(GrammarSymbol):
    def __init__(self, value):
        GrammarSymbol.__init__(self)
        self.value = value

    def nud(self, parser):
        return self.value


class Identifier(GrammarSymbol):
    def __init__(self, name, env):
        GrammarSymbol.__init__(self)
        self.value = env.formula_manager.get_symbol(name)
        if self.value is None:
            raise UndefinedSymbolError(name)

    def nud(self, parser):
        return self.value


class ExprIf(GrammarSymbol):
    def __init__(self):
        GrammarSymbol.__init__(self)
        self.lbp = 5

    def led(self, parser, left):
        cond_ = left
        then_ = parser.expression(self.lbp)
        parser.expect(ExprElse, ':')
        else_ = parser.expression(self.lbp)
        return parser.mgr.Ite(cond_, then_, else_)


class OpenArrayTypeTok(GrammarSymbol):
    def __init__(self):
        GrammarSymbol.__init__(self)
        self.lbp = 5

    def nud(self, parser):
        idx_type = parser.expression()
        parser.expect(ExprComma, ",")
        el_type = parser.expression()
        parser.expect(CloseBrace, "}")
        if type(parser.token) == OpenPar:
            parser.advance()
            default = parser.expression()
            parser.expect(ClosePar, ")")
            return parser.mgr.Array(idx_type, default)
        else:
            return types.ArrayType(idx_type, el_type)


class OpenPar(GrammarSymbol):
    def __init__(self):
        GrammarSymbol.__init__(self)
        self.lbp = 200

    def nud(self, parser):
        r = parser.expression()
        if type(parser.token) != ClosePar:
            raise SyntaxError("Expected ')', got '%s'" % parser.token)
        parser.advance()
        return r

    def led(self, parser, left):
        # a function call
        fun = left
        params = []
        if type(parser.token) != ClosePar:
            while True:
                r = parser.expression()
                params.append(r)
                if type(parser.token) != ExprComma:
                    break
                parser.advance()
        parser.expect(ClosePar, ")")
        return parser.mgr.Function(fun, params)


class OpenBrak(GrammarSymbol):
    def __init__(self):
        GrammarSymbol.__init__(self)
        self.lbp = 300

    def led(self, parser, left):
        # BVExtract, Select or Store
        op = left
        e1 = parser.expression()
        if type(parser.token) == ExprElse:
            #BVExtract
            parser.advance()
            end = parser.expression()
            parser.expect(CloseBrak, "]")
            return parser.mgr.BVExtract(op,
                                        e1.constant_value(),
                                        end.constant_value())
        elif type(parser.token) == CloseBrak:
            # Select
            parser.advance()
            return parser.mgr.Select(op, e1)
        elif type(parser.token) == ArrStore:
            #Store
            parser.advance()
            e2 = parser.expression()
            parser.expect(CloseBrak, "]")
            return parser.mgr.Store(op, e1, e2)
        else:
            raise PysmtSyntaxError("Unexpected token:" + str(parser.token))


class Quantifier(GrammarSymbol):
    def __init__(self, operator, lbp):
        GrammarSymbol.__init__(self)
        self.operator = operator
        self.lbp = lbp

    def nud(self, parser):
        qvars = []
        if type(parser.token) != ExprDot:
            while True:
                r = parser.expression()
                qvars.append(r)
                if type(parser.token) != ExprComma:
                    break
                parser.advance()
        parser.expect(ExprDot, ".")
        matrix = parser.expression(self.lbp)
        return self.operator(qvars, matrix)


class PrattParser(object):
    """The Pratt-Parser

    This parser is explained in:
       http://effbot.org/zone/simple-top-down-parsing.htm

    The LexerClass is required, and is the one doing the heavy lifting.
    """

    def __init__(self, LexerClass, env=None):
        if env is None:
            env = get_env()

        self.env = env
        self.mgr = env.formula_manager
        self.get_type = env.stc.get_type
        self.lexer = LexerClass(env)

        self.token = None
        self.tokenizer = None

    def expression(self, rbp=0):
        """Parses an expression"""
        t = self.token
        self.token = next(self.tokenizer)
        left = t.nud(self)
        while rbp < self.token.lbp:
            t = self.token
            self.token = next(self.tokenizer)
            left = t.led(self, left)
        return left

    def parse_fname(self, fname):
        """Parses the content of the given file name"""
        with open(fname, "r") as f:
            return self.parse(f.read())

    def parse(self, string):
        """Parses the content of the given string"""
        self.token = None
        self.tokenizer = self.lexer.tokenize(string)
        self.token = next(self.tokenizer)
        result = self.expression()
        try:
            bd = next(self.tokenizer)
            raise PysmtSyntaxError("Bogus data after expression: '%s' "
                                   "(Partial: %s)" % (bd, result))
        except StopIteration:
            return result

    def advance(self):
        """Advance reading of one token"""
        self.token = next(self.tokenizer)


    def expect(self, token_class, token_repr):
        """
        Check that the next token is the specified one or fail with a
        ParserError
        """
        if type(self.token) != token_class:
            raise PysmtSyntaxError("Expected '%s'" % token_repr)
        self.advance()

# EOC PrattParser

#
# Adapters
#
# These are adapters used to create tokens for various types of symbols
#

class EndOfInput(GrammarSymbol):
    pass


class UnaryOpAdapter(GrammarSymbol):
    """Adapter for unary operator."""

    def __init__(self, operator, lbp):
        GrammarSymbol.__init__(self)
        self.operator = operator
        self.lbp = lbp

    def nud(self, parser):
        right = parser.expression(self.lbp)
        return self.operator(right)


class InfixOpAdapter(GrammarSymbol):
    """Adapter for infix operator."""

    def __init__(self, operator, lbp):
        GrammarSymbol.__init__(self)
        self.operator = operator
        self.lbp = lbp

    def led(self, parser, left):
        right = parser.expression(self.lbp)
        return self.operator(left, right)

    def __repr__(self):
        return repr(self.operator)


class InfixOrUnaryOpAdapter(GrammarSymbol):
    """Adapter for operators that can be both binary infix or unary.

    b_operator and b_lbp: define the behavior when considering the symbol
                          as a binary operator.
    u_operator and u_lbp: define the behavior when considering the symbol
                          as a unary operator.
    """

    def __init__(self, b_operator, u_operator, b_lbp, u_lbp):
        GrammarSymbol.__init__(self)
        self.b_operator = b_operator
        self.u_operator = u_operator
        self.lbp = b_lbp
        self.u_lbp = u_lbp

    def nud(self, parser):
        right = parser.expression(self.u_lbp)
        return self.u_operator(right)

    def led(self, parser, left):
        right = parser.expression(self.lbp)
        return self.b_operator(left, right)


class FunctionCallAdapter(GrammarSymbol):
    """Adapter for function calls."""

    def __init__(self, operator, lbp):
        GrammarSymbol.__init__(self)
        self.operator = operator
        self.lbp = lbp

    def nud(self, parser):
        parser.advance() # OpenPar
        params = []
        if type(parser.token) != ClosePar:
            while True:
                r = parser.expression()
                params.append(r)
                if type(parser.token) != ExprComma:
                    break
                parser.advance()
        if type(parser.token) != ClosePar:
            raise SyntaxError("Expected ')'")
        parser.advance()
        return self.operator(*params)

    def __repr__(self):
        return repr(self.operator)


class ClosePar(GrammarSymbol):
    pass


class CloseBrak(GrammarSymbol):
    pass


class ExprElse(GrammarSymbol):
    pass


class ExprDot(GrammarSymbol):
    pass


class ExprComma(GrammarSymbol):
    pass



#
# This file is part of pySMT.
#
#   Copyright 2015 Andrea Micheli and Marco Gario
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
from pysmt.test import TestCase
from pysmt.type_checker import SimpleTypeChecker
from pysmt.printers import HRPrinter, HRSerializer
from pysmt.shortcuts import get_env, Symbol, reset_env
from pysmt.exceptions import UnsupportedOperatorError
from pysmt.environment import Environment
from pysmt.operators import new_node_type, all_types


class TestExtendSuper(TestCase):
    def test_new_node_type(self):
        old = list(all_types())
        idx = new_node_type(node_str="xor")
        self.assertIsNotNone(idx)
        self.assertNotIn(idx, old)
        with self.assertRaises(AssertionError):
            new_node_type(idx)
        XOR = idx

        # Ad-hoc method to handle printing of the new node
        def hrprinter_walk_xor(self, formula):
            self.stream.write("(")
            yield formula.arg(0)
            self.stream.write(" *+* ")
            yield formula.arg(1)
            self.stream.write(")")

        SimpleTypeChecker.walk_xor = SimpleTypeChecker.walk_bool_to_bool
        HRPrinter.walk_xor = hrprinter_walk_xor

        # Reset the env to recreate the TypeChecker and HRPrinter
        reset_env()

        # Shortcuts for function in env
        create_node = get_env().formula_manager.create_node
        # Create a test node (This implicitly calls the Type-checker)
        x = Symbol("x")
        f1 = create_node(node_type=XOR, args=(x,x))
        self.assertIsNotNone(f1)

        # String conversion should use the function defined above.
        s_f1 = str(f1)
        self.assertEqual(s_f1, "(x *+* x)")

        # We did not define an implementation for the Simplifier
        with self.assertRaises(UnsupportedOperatorError):
            f1.simplify()

        # Clean-up
        del SimpleTypeChecker.walk_xor
        del HRPrinter.walk_xor

        class MySimpleTypeChecker(SimpleTypeChecker):
            walk_xor = SimpleTypeChecker.walk_bool_to_bool

        class MyHRPrinter(HRPrinter):
            def walk_xor(self, formula):
                return self.walk_nary(formula, " *+* ")

        class MyHRSerializer(HRSerializer):
            PrinterClass = MyHRPrinter

        class MyEnv(Environment):
            TypeCheckerClass = MySimpleTypeChecker
            HRSerializerClass = MyHRSerializer

        with MyEnv() as myenv:
            create_node = myenv.formula_manager.create_node
            # Create a test node (This implicitly calls the Type-checker)
            x = Symbol("x")
            f1 = create_node(node_type=XOR, args=(x,x))
            self.assertIsNotNone(f1)

            # String conversion should use the function defined above.
            s_f1 = str(f1)
            self.assertEqual(s_f1, "(x *+* x)")

            # We did not define an implementation for the Simplifier
            with self.assertRaises(UnsupportedOperatorError):
                f1.simplify()

        return



#
# This file is part of pySMT.
#
#   Copyright 2014 Andrea Micheli and Marco Gario
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
import os
import pytest

from pysmt.shortcuts import Implies, is_sat, reset_env, Symbol, Iff
from pysmt.rewritings import CNFizer
from pysmt.logics import QF_BOOL, QF_LRA, QF_LIA, QF_UFLIRA
from pysmt.test import TestCase, skipIfNoSolverForLogic, main
from pysmt.test.examples import get_example_formulae
from pysmt.test.smtlib.parser_utils import SMTLIB_TEST_FILES, SMTLIB_DIR
from pysmt.smtlib.parser import get_formula_fname

class TestCnf(TestCase):

    def do_examples(self, logic):
        conv = CNFizer()
        for example in get_example_formulae():
            if example.logic != logic:
                continue
            cnf = conv.convert_as_formula(example.expr)

            self.assertValid(Implies(cnf, example.expr), logic=logic)

            res = is_sat(cnf, logic=logic)
            self.assertEqual(res, example.is_sat)


    @skipIfNoSolverForLogic(QF_BOOL)
    def test_examples_solving_bool(self):
        self.do_examples(QF_BOOL)

    @skipIfNoSolverForLogic(QF_LRA)
    def test_examples_solving_lra(self):
        self.do_examples(QF_LRA)

    @skipIfNoSolverForLogic(QF_LIA)
    def test_examples_solving_lia(self):
        self.do_examples(QF_LIA)

    @skipIfNoSolverForLogic(QF_LIA)
    def test_smtlib_cnf_small(self):
        cnt = 0
        max_cnt = 3
        for (logic, f, expected_result) in SMTLIB_TEST_FILES:
            if logic != QF_LIA:
                continue
            self._smtlib_cnf(f, logic, expected_result)
            cnt += 1
            if cnt == max_cnt:
                break

    @pytest.mark.slow
    @skipIfNoSolverForLogic(QF_UFLIRA)
    def test_smtlib_cnf(self):
        for (logic, f, expected_result) in SMTLIB_TEST_FILES:
            if logic != QF_UFLIRA:
                continue
            self._smtlib_cnf(f, logic, expected_result)

    def _smtlib_cnf(self, filename, logic, res_is_sat):
        reset_env()
        conv = CNFizer()
        smtfile = os.path.join(SMTLIB_DIR, filename)
        assert os.path.exists(smtfile)

        expr = get_formula_fname(smtfile)
        if not logic.quantifier_free:
            with self.assertRaises(NotImplementedError):
                conv.convert_as_formula(expr)
            return

        cnf = conv.convert_as_formula(expr)
        self.assertValid(Implies(cnf, expr), logic=logic)

        res = is_sat(cnf, logic=logic)
        self.assertEqual(res, res_is_sat)

    @skipIfNoSolverForLogic(QF_BOOL)
    def test_implies(self):
        a,b,c,d = (Symbol(x) for x in "abcd")
        f = Implies(Iff(a, b), Iff(c, d))

        conv = CNFizer()
        cnf = conv.convert_as_formula(f)

        self.assertValid(Implies(cnf, f), logic=QF_BOOL)

if __name__ == '__main__':
    main()



#
# This file is part of pySMT.
#
#   Copyright 2014 Andrea Micheli and Marco Gario
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
"""Defines constants for the commands of the SMT-LIB"""

ASSERT='assert'
CHECK_SAT='check-sat'
CHECK_SAT_ASSUMING='check-sat-assuming'
DECLARE_CONST='declare-const'
DECLARE_FUN='declare-fun'
DECLARE_SORT='declare-sort'
DEFINE_FUN='define-fun'
DEFINE_FUN_REC='define-fun-rec'
DEFINE_FUNS_REC='define-funs-rec'
DEFINE_SORT='define-sort'
ECHO='echo'
EXIT='exit'
GET_ASSERTIONS='get-assertions'
GET_ASSIGNMENT='get-assignment'
GET_INFO='get-info'
GET_MODEL='get-model'
GET_OPTION='get-option'
GET_PROOF='get-proof'
GET_UNSAT_ASSUMPTIONS='get-unsat-assumptions'
GET_UNSAT_CORE='get-unsat-core'
GET_VALUE='get-value'
POP='pop'
PUSH='push'
RESET='reset'
RESET_ASSERTIONS='reset-assertions'
SET_INFO='set-info'
SET_LOGIC='set-logic'
SET_OPTION='set-option'

#

SMT_LIB_2_0 = [
    SET_LOGIC,
    SET_OPTION,
    SET_INFO,
    DECLARE_SORT,
    DEFINE_SORT,
    DECLARE_FUN,
    DEFINE_FUN,
    PUSH,
    POP,
    ASSERT,
    CHECK_SAT,
    GET_ASSERTIONS,
    GET_VALUE,
    GET_MODEL,
    GET_PROOF,
    GET_UNSAT_CORE,
    GET_INFO,
    GET_OPTION,
    EXIT,
]

SMT_LIB_2_5 = SMT_LIB_2_0 + [
    CHECK_SAT_ASSUMING,
    DECLARE_CONST,
    DEFINE_FUN_REC,
    DEFINE_FUNS_REC,
    ECHO,
    GET_ASSIGNMENT,
    GET_UNSAT_ASSUMPTIONS,
    RESET,
    RESET_ASSERTIONS,
]

ALL_COMMANDS = SMT_LIB_2_5



#
# This file is part of pySMT.
#
#   Copyright 2014 Andrea Micheli and Marco Gario
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
import pytest

from pysmt.test import TestCase, skipIfSolverNotAvailable, main
from pysmt.test.examples import get_example_formulae
from pysmt.environment import get_env
from pysmt.shortcuts import (Array, Store, Int, Iff, Symbol, Plus, Equals, And,
                             Real, Times, Not, FALSE, Or, TRUE)
from pysmt.typing import INT, REAL
from pysmt.simplifier import BddSimplifier
from pysmt.logics import QF_BOOL
from pysmt.exceptions import ConvertExpressionError, NoSolverAvailableError


class TestSimplify(TestCase):

    @pytest.mark.slow
    @skipIfSolverNotAvailable("z3")
    @skipIfSolverNotAvailable("cvc5")
    def test_simplify_qf(self):
        simp = get_env().simplifier
        for (f, _, _, logic) in get_example_formulae():
            if logic.is_quantified(): continue
            sname = "z3" if not logic.theory.strings else "cvc5"
            simp.validate_simplifications = sname
            sf = f.simplify()
            simp.validate_simplifications = None
            self.assertValid(Iff(f, sf), solver_name=sname,
                             msg="Simplification did not provide equivalent "+
                                "result:\n f= %s\n sf = %s" % (f, sf))

    @pytest.mark.slow
    @skipIfSolverNotAvailable("z3")
    def test_simplify_q(self):
        simp = get_env().simplifier
        for (f, _, _, logic) in get_example_formulae():
            if logic.quantifier_free: continue
            simp.validate_simplifications = "z3"
            sf = f.simplify()
            simp.validate_simplifications = None
            self.assertValid(Iff(f, sf), solver_name='z3',
                             msg="Simplification did not provide equivalent "+
                            "result:\n f= %s\n sf = %s" % (f, sf))


    def test_array_value(self):
        a1 = Array(INT, Int(0), {Int(12) : Int(10)})
        a2 = Store(Array(INT, Int(0)), Int(12), Int(10))
        self.assertEqual(a1, a2.simplify())

    @skipIfSolverNotAvailable("bdd")
    def test_bdd_simplify(self):
        s = BddSimplifier()
        for (f, _, _, logic) in get_example_formulae():
            if logic == QF_BOOL:
                fprime = s.simplify(f)
                self.assertValid(Iff(fprime, f))

        s = BddSimplifier()
        try:
            s.validate_simplifications = True
        except ValueError:
            self.assertTrue(len(self.env.factory.all_solvers())==1)
        for (f, _, _, logic) in get_example_formulae():
            if logic == QF_BOOL:
                fprime = s.simplify(f)
                self.assertValid(Iff(fprime, f))

    @skipIfSolverNotAvailable("bdd")
    @skipIfSolverNotAvailable("z3")
    def test_bdd_simplify_bool_abs(self):
        s = BddSimplifier()
        for (f, _, _, logic) in get_example_formulae():
            if not logic.theory.linear: continue
            if logic != QF_BOOL:
                with self.assertRaises(ConvertExpressionError):
                    s.simplify(f)

        s = BddSimplifier(bool_abstraction=True)
        for (f, _, _, logic) in get_example_formulae():
            if logic.quantifier_free:
                fprime = s.simplify(f)
                try:
                    self.assertValid(Iff(fprime, f))
                except NoSolverAvailableError:
                    pass

        s = BddSimplifier(bool_abstraction=True)
        f = And(Equals(Plus(Int(5), Int(1)),
                       Int(6)),
                Symbol("x"))
        fs = s.simplify(f)
        self.assertEqual(fs, Symbol("x"))


    def test_times_one(self):
        r = Symbol("r", REAL)
        f = Times(r, r, Real(1))
        f = f.simplify()
        self.assertNotIn(Real(1), f.args())

    @skipIfSolverNotAvailable("z3")
    def test_plus_negatives(self):
        r0 = Symbol("r0", REAL)
        r1 = Symbol("r1", REAL)
        p_1 = Real(1)
        m_1 = Real(-1)
        p_2 = Real(2)
        m_4 = Real(-4)

        # 4 * r0 + (-1) * r1 + 2 - 4
        neg_r1 = Times(m_1, r1)
        m_4_r0 = Times(Real(4), r0)
        expr = Plus(m_4_r0, neg_r1, p_2, m_4)
        res = expr.simplify()
        self.assertValid(Equals(expr, res))
        stack = [res]
        while stack:
            curr = stack.pop()
            if curr.is_plus():
                stack.extend(curr.args())
            elif curr.is_minus():
                stack.extend(curr.args())
            elif curr.is_times():
                stack.extend(curr.args())
            elif curr.is_constant():
                self.assertNotEqual(curr, m_1)
                self.assertNotEqual(curr, p_1)
            elif not curr.is_symbol():
                # unexpected expression type.
                self.assertTrue(False)

    @skipIfSolverNotAvailable("z3")
    def test_sum_all_negatives(self):
        r0 = Symbol("r0", REAL)
        r1 = Symbol("r1", REAL)
        m_1 = Real(-1)

        # -4 * r0 + (-1) * r1
        neg_r1 = Times(m_1, r1)
        m_4_r0 = Times(Real(-4), r0)
        expr = Plus(m_4_r0, neg_r1)
        res = expr.simplify()
        self.assertValid(Equals(expr, res))

    @skipIfSolverNotAvailable("z3")
    def test_plus_algebraic(self):
        from pysmt.constants import Numeral
        env = get_env()
        mgr = env.formula_manager
        r0 = Symbol("r0", REAL)
        p_2 = Real(2)
        m_5 = mgr._Algebraic(Numeral(-5))
        m_3 = mgr._Algebraic(Numeral(-3))

        # r0 + 2 - 5
        expr = Plus(r0, p_2, m_5)
        res = expr.simplify()
        self.assertValid(Equals(expr, res))
        self.assertIn(m_3, res.args())

    @skipIfSolverNotAvailable("z3")
    def test_times_algebraic(self):
        from pysmt.constants import Numeral
        env = get_env()
        mgr = env.formula_manager
        r0 = Symbol("r0", REAL)
        p_2 = Real(2)
        m_5 = mgr._Algebraic(Numeral(-5))
        m_10 = mgr._Algebraic(Numeral(-10))

        # -5 * r0 * 2
        expr = Times(m_5, r0, p_2)
        res = expr.simplify()
        self.assertValid(Equals(expr, res))
        self.assertIn(m_10, res.args())


    def test_and_flattening(self):
        x,y,z = (Symbol(name) for name in "xyz")
        f1 = And(x, y, z)
        f2 = And(x, And(y, z))
        self.assertEqual(f2.simplify(), f1)

    def test_or_flattening(self):
        x,y,z = (Symbol(name) for name in "xyz")
        f1 = Or(x, y, z)
        f2 = Or(x, Or(y, z))
        self.assertEqual(f2.simplify(), f1)

    def test_trivial_false_and(self):
        x,y,z = (Symbol(name) for name in "xyz")
        f = And(x, y, z, Not(x))
        self.assertEqual(f.simplify(), FALSE())

    def test_trivial_true_or(self):
        x,y,z = (Symbol(name) for name in "xyz")
        f = Or(x, y, z, Not(x))
        self.assertEqual(f.simplify(), TRUE())

    def test_trivial_true_times(self):
        x,y,z = (Symbol(name, REAL) for name in "xyz")
        f = Equals(Times(x, z, y), Times(z, y, x))
        self.assertEqual(f.simplify(), TRUE())

if __name__ == '__main__':
    main()



#
# This file is part of pySMT.
#
#   Copyright 2014 Andrea Micheli and Marco Gario
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#

import sys
import subprocess
import pkgutil
import os.path

from pysmt.test import TestCase, main
import pysmt


class TestImports(TestCase):

    EXCLUSION_LIST = ["pysmt.test", "pysmt.solvers", "pysmt.cmd"]

    def test_imports(self):
        stack = [(pysmt.__name__, pysmt.__path__)]
        while stack:
            module_name, module_path = stack.pop()
            for _, name, ispkg in pkgutil.iter_modules(module_path):
                fullname = "%s.%s" % (module_name, name)
                command = [sys.executable, '-c', 'import %s' % fullname]
                returncode = subprocess.call(command)
                self.assertEqual(returncode, 0, msg="Failed to import %s module" % fullname)
                if ispkg and fullname not in TestImports.EXCLUSION_LIST:
                    stack.append((fullname, [os.path.join(p, name) for p in module_path]))

if __name__ == '__main__':
    main()



# Portfolio solving makes it possible to run multiple solvers in
# parallel.  As soon as the first solver completes, all other solvers
# are stopped.  Current support for portfolio is mostly focused on
# one-shot calls, with only functional incrementality: the interface
# looks incremental, but internally there is no guarantee of re-using
# the solver state.
#
from pysmt.shortcuts import Symbol, Implies, TRUE, FALSE, Not

x, y = Symbol("x"), Symbol("y")
f = Implies(x, y)

# The first example shows how to use multiple solvers in the with the
# is_sat shortcut
#
from pysmt.shortcuts import is_sat

# We enable logging to see what is going on behind the scenes:
import logging
_info = logging.getLogger(__name__).info
logging.basicConfig(level=logging.DEBUG)

# A solver set is an iterable of solver names or pairs of
# solvers+options (See next example)

_info("Example 1: is_sat")
solvers_set = ["z3", "yices", "msat"]
res = is_sat(f, portfolio=solvers_set)
assert res is True

# Behind the scenes, pySMT launched 3 processes and solved the
# expression in parallel.
#
# The is_sat shortcut is useful for prototyping and exploration, but
# we typically need more control over the solver. The Portfolio class
# behaves as a solver and as such implements most of the methods of a
# regular solver.
from pysmt.shortcuts import Portfolio
from pysmt.logics import QF_UFLIRA

# The options given to the Portfolio will be passed to all solvers, in
# particular, we are enabling incrementality and model generation.

_info("Example 2: Portfolio()")
with Portfolio(solvers_set,
               logic=QF_UFLIRA,
               incremental=True,
               generate_models=True) as s:
    s.add_assertion(f)
    s.push()
    s.add_assertion(x)
    res = s.solve()
    v_y = s.get_value(y)
    assert v_y is TRUE()

    s.pop()
    s.add_assertion(Not(y))
    res = s.solve()
    v_x = s.get_value(x)
    assert v_x is FALSE()

# Portfolio can also be useful to tweak heuristics of the solvers.
# For supported solver options, please refer to the solver
# documentation. This is an area of pySMT that could use additional
# feedback and help!

_info("Example 3: Portfolio w options")
with Portfolio([("msat", {"random_seed": 1}),
                ("msat", {"random_seed": 17}),
                ("msat", {"random_seed": 42}),
                "cvc5", "yices"],
               logic=QF_UFLIRA,
               incremental=False,
               generate_models=False) as s:
    res = s.is_sat(f)
    assert res is True



# This example requires a solver that supports QF_AUFLIRA
#  e.g. MathSat or Z3
#
# Theory combination is the possibility of mixing theories with an SMT
# problem. This goes beyond having subexpressions belonging to
# different theories, and really requires mixing theories in the same
# expression.
#
# In this example, we use four theories supported by pySMT:
# BitVectors, Integers, Reals, and Arrays.
#
#
from pysmt.shortcuts import Symbol, BV, Real, And, BVToNatural, ToReal
from pysmt.shortcuts import get_model
from pysmt.typing import BV8, REAL, INT, ArrayType

# We create a map from BitVectors to Reals, so that each bitvector
# value (interpreted as unary number) is equal to the Real
# value.
#
# The map is represented by an Array of type BV8 -> Real
map_type = ArrayType(BV8, REAL)
my_map = Symbol("my_map", map_type)

# Fill-up the map, by defining all 256 values:
for i in range(0, 255):
    my_map = my_map.Store(BV(i, 8), Real(i))

# We want to find find a value for which our relation does not work.
# In other words, we ask if there is a value for the bitvector
# s.t. the corresponding value in the array is different from the
# unary interpretation of the bitvector.
bv_var = Symbol("bv", BV8)
int_var = Symbol("int", INT)
real_var = Symbol("real", REAL)

f = And(
    # Convert the BV into INT
    int_var.Equals(BVToNatural(bv_var)),
    # Convert the INT into REAL
    real_var.Equals(ToReal(int_var)),
    # Compare the value stored in the map with the REAL value
    my_map.Select(bv_var).NotEquals(real_var)
    )

print(get_model(f)) # Indeed our range only gets up to 254!!!



#
# This file is part of pySMT.
#
#   Copyright 2014 Andrea Micheli and Marco Gario
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
from pysmt.exceptions import SolverAPINotFound

try:
    import repycudd
except ImportError:
    raise SolverAPINotFound

import pysmt.logics
from pysmt import typing as types
from pysmt.solvers.solver import Solver, Converter, SolverOptions
from pysmt.solvers.eager import EagerModel
from pysmt.walkers import DagWalker
from pysmt.decorators import clear_pending_pop, catch_conversion_error
from pysmt.exceptions import (ConvertExpressionError, PysmtValueError,
                              PysmtTypeError)
from pysmt.oracles import get_logic
from pysmt.solvers.qelim import QuantifierEliminator


class BddOptions(SolverOptions):
    """Options for the BDD Solver.

    * static_ordering: None, list of Symbols
      Enable static ordering with the given list of symbols

    * dynamic_reordering: True, False
      Enable dynamic reordering

    * reordering_algorithm: BddOptions.CUDD_ALL_REORDERING_ALGORITHMS
      Specify which reordering algorithm to use when dynamic_reordering
      is enabled.

    """
    ## CUDD Reordering algorithms
    CUDD_ALL_REORDERING_ALGORITHMS = range(1,23)

    CUDD_REORDER_SAME, \
    CUDD_REORDER_NONE, \
    CUDD_REORDER_RANDOM, \
    CUDD_REORDER_RANDOM_PIVOT, \
    CUDD_REORDER_SIFT, \
    CUDD_REORDER_SIFT_CONVERGE, \
    CUDD_REORDER_SYMM_SIFT, \
    CUDD_REORDER_SYMM_SIFT_CONV, \
    CUDD_REORDER_WINDOW2, \
    CUDD_REORDER_WINDOW3, \
    CUDD_REORDER_WINDOW4, \
    CUDD_REORDER_WINDOW2_CONV, \
    CUDD_REORDER_WINDOW3_CONV, \
    CUDD_REORDER_WINDOW4_CONV, \
    CUDD_REORDER_GROUP_SIFT, \
    CUDD_REORDER_GROUP_SIFT_CONV, \
    CUDD_REORDER_ANNEALING, \
    CUDD_REORDER_GENETIC, \
    CUDD_REORDER_LINEAR, \
    CUDD_REORDER_LINEAR_CONVERGE, \
    CUDD_REORDER_LAZY_SIFT, \
    CUDD_REORDER_EXACT = CUDD_ALL_REORDERING_ALGORITHMS

    def __init__(self, **base_options):
        SolverOptions.__init__(self, **base_options)

        if self.random_seed is not None:
            raise PysmtValueError("'random_seed' option not supported.")
        if self.unsat_cores_mode is not None:
            raise PysmtValueError("'unsat_cores_mode' option not supported.")

        for k,v in self.solver_options.items():
            if k == "static_ordering":
                if v is not None:
                    try:
                        valid = all(x.is_symbol(types.BOOL) for x in v)
                    except:
                        valid = False
                    if not valid:
                        raise PysmtValueError("The BDD static ordering must be" \
                                              " a list of Boolean variables")
            elif k == "dynamic_reordering":
                if v not in (True, False):
                    raise PysmtValueError("Invalid value %s for '%s'" % \
                                          (str(k),str(v)))
            elif k == "reordering_algorithm":
                if v not in BddOptions.CUDD_ALL_REORDERING_ALGORITHMS:
                    raise PysmtValueError("Invalid value %s for '%s'" % \
                                          (str(k),str(v)))
            else:
                raise PysmtValueError("Unrecognized option '%s'." % k)
            # Store option
            setattr(self, k, v)

        # Set Defaults
        if not hasattr(self, "dynamic_reordering"):
            self.dynamic_reordering = False
        if not hasattr(self, "reordering_algorithm"):
            if not self.dynamic_reordering:
                self.reordering_algorithm = None
            else:
                self.reordering_algorithm = BddOptions.CUDD_REORDER_SIFT
        if not hasattr(self, "static_ordering"):
            self.static_ordering = None

        # Consistency check
        if not self.dynamic_reordering and self.reordering_algorithm is not None:
            raise PysmtValueError("reordering_algorithm requires "
                                  "dynamic_reordering.")

    def __call__(self, solver):
        # Impose initial ordering
        if self.static_ordering is not None:
            for var in self.static_ordering:
                solver.declare_variable(var)
        if self.dynamic_reordering:
            solver.ddmanager.AutodynEnable(self.reordering_algorithm)
        else:
            solver.ddmanager.AutodynDisable()

# EOC BddOptions


class BddSolver(Solver):
    """CUDD BDD Solver"""
    LOGICS = [ pysmt.logics.QF_BOOL, pysmt.logics.BOOL ]
    OptionsClass = BddOptions

    def __init__(self, environment, logic, **options):
        Solver.__init__(self,
                        environment=environment,
                        logic=logic,
                        **options)

        self.mgr = environment.formula_manager
        self.ddmanager = repycudd.DdManager()
        self.converter = BddConverter(environment=self.environment,
                                      ddmanager=self.ddmanager)
        self.options(self)

        # This stack keeps a pair (Expr, Bdd), with the semantics that
        # the bdd at the i-th element of the list contains the bdd of
        # the conjunction of all previous expressions.
        # The construction of the Bdd is done during solve()
        self.assertions_stack = None
        self.reset_assertions() # Initialize the stack

        self.backtrack = []
        self.latest_model = None

    @clear_pending_pop
    def reset_assertions(self):
        true_formula = self.mgr.Bool(True)
        self.assertions_stack = [(true_formula,
                                  self.converter.convert(true_formula))]

    @clear_pending_pop
    def declare_variable(self, var):
        self.converter.declare_variable(var)

    @clear_pending_pop
    def add_assertion(self, formula, named=None):
        self.assertions_stack.append((formula, None))

    @clear_pending_pop
    def solve(self, assumptions=None):
        if assumptions is not None:
            self.push()
            self.add_assertion(self.mgr.And(assumptions))
            self.pending_pop = True

        for (i, (expr, bdd)) in enumerate(self.assertions_stack):
            if bdd is None:
                bdd_expr = self.converter.convert(expr)
                _, previous_bdd = self.assertions_stack[i-1]
                new_bdd = self.ddmanager.And(previous_bdd, bdd_expr)
                self.assertions_stack[i] = (expr, new_bdd)

        _, current_state = self.assertions_stack[-1]
        res = (current_state != self.ddmanager.Zero())
        # Invalidate cached model
        self.latest_model = None
        return res

    def get_value(self, item):
        if self.latest_model is None:
            self.get_model()
        return self.latest_model.get_value(item)

    def get_model(self):
        # TODO: We could provide a more sophisticated Model class,
        # that would contain the current Bdd and a copy of the
        # DdManager. This would make it possible to apply other
        # operations on the model (e.g., enumeration) in a simple way.
        if self.latest_model is None:
            _, current_state = self.assertions_stack[-1]
            assert current_state is not None, "solve() should be called before get_model()"
            # Build ddArray of variables
            var_array = self.converter.get_all_vars_array()
            minterm_set = self.ddmanager.PickOneMinterm(current_state,
                                                        var_array,
                                                        len(var_array))
            minterm = next(repycudd.ForeachCubeIterator(self.ddmanager,
                                                        minterm_set))
            assignment = {}
            for i, node in enumerate(var_array):
                value = self.mgr.Bool(minterm[i] == 1)
                key = self.converter.idx2var[node.NodeReadIndex()]
                assignment[key] = value

            self.latest_model = EagerModel(assignment=assignment,
                                           environment=self.environment)
        return self.latest_model

    @clear_pending_pop
    def push(self, levels=1):
        for _ in range(levels):
            self.backtrack.append(len(self.assertions_stack))

    @clear_pending_pop
    def pop(self, levels=1):
        for _ in range(levels):
            l = self.backtrack.pop()
            self.assertions_stack = self.assertions_stack[:l]

    def _exit(self):
        del self.ddmanager


class BddConverter(Converter, DagWalker):

    def __init__(self, environment, ddmanager):
        DagWalker.__init__(self)

        self.environment = environment
        self.fmgr = self.environment.formula_manager
        self.ddmanager = ddmanager
        # Note: Nodes in repycudd are not shared, but they overload all
        # methods to perform comparison. This means that for two
        # wrappers for the variable x, we have that id(x) != id(x1)
        # but x == x1.  Nevertheless, we need to store the ids, since
        # nodes can be moved during operations by the
        # ddManager. Therefore, when using nodes in a map, we should
        # use the ids.
        self.idx2var = {}
        self.var2node = {}
        self.back_memoization = {}

    @catch_conversion_error
    def convert(self, formula):
        """Convert a PySMT formula into a BDD."""
        return self.walk(formula)

    def back(self, bdd_expr):
        return self._walk_back(bdd_expr, self.fmgr).simplify()

    def get_all_vars_array(self):
        # NOTE: This way of building the var_array does not look
        #       robust.  There might be an issue if variables are
        #       added and the order of enumeration of the dictionary
        #       changes and we rely on this order outside of this class.
        var_array = repycudd.DdArray(self.ddmanager, len(self.idx2var))
        for i, node_idx in enumerate(self.idx2var):
            var_array[i] = self.ddmanager[node_idx]
        return var_array

    def cube_from_var_list(self, var_list):
        indices = repycudd.IntArray(len(var_list))
        for i, v in enumerate(var_list):
            indices[i] = self.var2node[v].NodeReadIndex()
        cube = self.ddmanager.IndicesToCube(indices, len(var_list))
        return cube

    def declare_variable(self, var):
        if not var.is_symbol(type_=types.BOOL):
            raise PysmtTypeError("Trying to declare as a variable something "
                                 "that is not a symbol: %s" % var)
        if var not in self.var2node:
            node = self.ddmanager.NewVar()
            self.idx2var[node.NodeReadIndex()] = var
            self.var2node[var] = node

    def walk_and(self, formula, args, **kwargs):
        res = args[0]
        for a in args[1:]:
            res = self.ddmanager.And(a, res)
        return res

    def walk_or(self, formula, args, **kwargs):
        res = args[0]
        for a in args[1:]:
            res = self.ddmanager.Or(res,a)
        return res

    def walk_not(self, formula, args, **kwargs):
        return self.ddmanager.Not(args[0])

    def walk_symbol(self, formula, **kwargs):
        if not formula.is_symbol(types.BOOL):
            raise ConvertExpressionError("Cannot handle given type: %s" % str(formula))
        if formula not in self.var2node:
            self.declare_variable(formula)
        res = self.var2node[formula]
        return res

    def walk_implies(self, formula, args, **kwargs):
        lhs, rhs = args
        return self.ddmanager.Or(lhs.Not(), rhs)

    def walk_iff(self, formula, args, **kwargs):
        lhs, rhs = args
        pos = self.ddmanager.And(lhs, rhs)
        neg = self.ddmanager.And(lhs.Not(), rhs.Not())
        return self.ddmanager.Or(pos, neg)

    def walk_forall(self, formula, args, **kwargs):
        f = args[0]
        cube = self.cube_from_var_list(formula.quantifier_vars())
        res = self.ddmanager.UnivAbstract(f, cube)
        return res

    def walk_exists(self, formula, args, **kwargs):
        f = args[0]
        cube = self.cube_from_var_list(formula.quantifier_vars())
        res = self.ddmanager.ExistAbstract(f, cube)
        return res

    def walk_bool_constant(self, formula, **kwargs):
        if formula.is_true():
            return self.ddmanager.One()
        else:
            return self.ddmanager.Zero()

    def _walk_back(self, bdd, mgr):
        stack = [bdd]
        while len(stack) > 0:
            current = stack.pop()
            # If the node haven't been explored yet, it is not in back_memoization,
            # If back_memoization contains None, we are exploring the children
            # Otherwise, it contains the pySMT expression corresponding to the node
            if current not in self.back_memoization:
                self.back_memoization[current] = None
                stack.append(current)
                if current != self.ddmanager.Zero() and \
                   current != self.ddmanager.One():
                    t = current.T()
                    e = current.E()
                    stack.append(t)
                    stack.append(e)
            elif self.back_memoization[current] is None:
                if current == self.ddmanager.Zero():
                    res = mgr.FALSE()
                elif current == self.ddmanager.One():
                    res = mgr.TRUE()
                else:
                    var = self.idx2var[current.NodeReadIndex()]
                    t = current.T()
                    e = current.E()
                    expr_t = self.back_memoization[t]
                    expr_e = self.back_memoization[e]

                    if current.IsComplement():
                        # (v /\ !t) \/ (!v /\ !e)
                        #     P            N
                        p = mgr.And(var, mgr.Not(expr_t))
                        n = mgr.And(mgr.Not(var), mgr.Not(expr_e))
                        res = mgr.Or(p, n)
                    else:
                        # (v /\ t) \/ (!v /\ e)
                        #    P             N
                        p = mgr.And(var, expr_t)
                        n = mgr.And(mgr.Not(var), expr_e)
                        res = mgr.Or(p, n)

                self.back_memoization[current] = res
            else:
                # we already visited the node, nothing else to do
                pass
        return self.back_memoization[bdd]

# EOC BddConverter


class BddQuantifierEliminator(QuantifierEliminator):

    LOGICS = [pysmt.logics.BOOL]

    def __init__(self, environment, logic=None):
        QuantifierEliminator.__init__(self)
        self.environment = environment
        self.logic = logic
        self.ddmanager = repycudd.DdManager()
        self.converter = BddConverter(environment=environment,
                                      ddmanager=self.ddmanager)

    def eliminate_quantifiers(self, formula):
        logic = get_logic(formula, self.environment)
        if not logic <= pysmt.logics.BOOL:
            raise NotImplementedError("BDD-based quantifier elimination only "\
                                      "supports pure-boolean formulae."\
                                      "(detected logic is: %s)" % str(logic))

        bdd = self.converter.convert(formula)
        pysmt_res = self.converter.back(bdd)
        return pysmt_res

    def _exit(self):
        del self.ddmanager



#
# This file is part of pySMT.
#
#   Copyright 2014 Andrea Micheli and Marco Gario
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
from fractions import Fraction as pyFraction

from pysmt.test import TestCase, main, skipIf
from pysmt.constants import Fraction, Integer, HAS_GMPY, USE_GMPY
from pysmt.constants import is_pysmt_fraction, is_pysmt_integer
from pysmt.constants import is_python_integer, is_python_boolean, is_python_rational
from pysmt.constants import pysmt_integer_from_integer
from pysmt.constants import to_python_integer
from pysmt.constants import pysmt_fraction_from_rational


class TestConstants(TestCase):

    def test_is_methods(self):
        self.assertFalse(is_pysmt_fraction(4))
        self.assertTrue(is_pysmt_fraction(Fraction(4)))

        self.assertFalse(is_pysmt_integer(4.0))
        self.assertTrue(is_pysmt_integer(Integer(4)))

        self.assertTrue(is_python_integer(int(2)))
        if HAS_GMPY:
            from gmpy2 import mpz
            self.assertTrue(is_python_integer(mpz(1)))

        if HAS_GMPY:
            from gmpy2 import mpz, mpq
            self.assertTrue(is_python_rational(mpz(1)))
            self.assertTrue(is_python_rational(mpq(1)))

        self.assertTrue(is_python_rational(pyFraction(5)))
        self.assertTrue(is_python_rational(3))

        self.assertTrue(is_python_boolean(True))
        self.assertTrue(is_python_boolean(False))

    def test_pysmt_integer_from_integer(self):
        self.assertEqual(Integer(4), pysmt_integer_from_integer(4))
        self.assertEqual(Integer(4), pysmt_integer_from_integer(Integer(4)))
        self.assertEqual(Integer(4), pysmt_integer_from_integer(Fraction(4)))
        self.assertEqual(Integer(4), pysmt_integer_from_integer(pyFraction(4)))

    def test_to_python_integer(self):
        res = int(4)
        self.assertEqual(res, to_python_integer(pysmt_integer_from_integer(4)))
        self.assertEqual(res, to_python_integer(pysmt_integer_from_integer(Integer(4))))
        self.assertEqual(res, to_python_integer(pysmt_integer_from_integer(Fraction(4))))
        self.assertEqual(res, to_python_integer(pysmt_integer_from_integer(pyFraction(4))))

    def test_pysmt_fraction_from_rational(self):
        self.assertEqual(Fraction(4,5), pysmt_fraction_from_rational("4/5"))
        self.assertEqual(Fraction(4,5), pysmt_fraction_from_rational(pyFraction(4,5)))
        self.assertEqual(Fraction(4,5), pysmt_fraction_from_rational(Fraction(4,5)))
        self.assertEqual(Fraction(4), pysmt_fraction_from_rational(4))
        self.assertEqual(Fraction(4), pysmt_fraction_from_rational(Integer(4)))

    @skipIf(not HAS_GMPY or USE_GMPY, "Not using GMPY")
    def test_constructors_from_gmpy(self):
        from gmpy2 import mpz

        mgr = self.env.formula_manager
        five = mpz(5)
        six = mpz(6)

        int5a = mgr.Int(5)
        int5b = mgr.Int(five)
        type_5a = type(int5a.constant_value())
        type_5b = type(int5b.constant_value())
        self.assertEqual(type_5a, type_5b)

        # Note that we first create the constant using the GMPY value
        int6b = mgr.Int(six)
        int6a = mgr.Int(6)

        type_6a = type(int6a.constant_value())
        type_6b = type(int6b.constant_value())
        self.assertEqual(type_6a, type_6b)
        # Verify that type is the same independently of how the first
        # node was created
        self.assertEqual(type_5a, type_6a)

        # Repeat with BV
        bv5a = mgr.BV(5, 8)
        bv5b = mgr.BV(five, 8)
        type_5a = type(bv5a.constant_value())
        type_5b = type(bv5b.constant_value())
        self.assertEqual(type_5a, type_5b)

        # Note that we first create the constant using the GMPY value
        bv6b = mgr.BV(six, 8)
        bv6a = mgr.BV(6, 8)

        type_6a = type(bv6a.constant_value())
        type_6b = type(bv6b.constant_value())
        self.assertEqual(type_6a, type_6b)
        # Verify that type is the same independently of how the first
        # node was created
        self.assertEqual(type_5a, type_6a)


if __name__ == '__main__':
    main()



#
# This file is part of pySMT.
#
#   Copyright 2014 Andrea Micheli and Marco Gario
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
import os

from pysmt.test import TestCase, main
from pysmt.smtlib.parser import SmtLibParser
from pysmt.exceptions import PysmtTypeError

class TestTypeError(TestCase):

    def test_wrong(self):
        d = os.path.dirname(os.path.realpath(__file__))
        smtfile = d + "/small_set/negative/wrong1.smt2.bz2"
        parser = SmtLibParser()
        with self.assertRaises(PysmtTypeError):
            parser.get_script_fname(smtfile)



if __name__ == '__main__':
    main()



# Copyright 2014 Andrea Micheli and Marco Gario
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import glob

from pysmt.cmd.installers.base import SolverInstaller


class MSatInstaller(SolverInstaller):

    SOLVER = "msat"

    def __init__(self, install_dir, bindings_dir, solver_version,
                 mirror_link=None):

        # Getting the right archive name
        archive_name_template = "mathsat-{version}-{os}-{arch}.{ext}"
        format = {
                "version": solver_version,
                "os" : self.os_name,
                "arch": self.architecture,
                "ext": "tar.gz"
        }
        if self.os_name == "windows":
            format["ext"] = "zip"
            format["arch"] = "msvc"
            format["os"] = "win64" if self.architecture == "x86_64" else "win32"
        elif self.os_name == "darwin":
            # Since version 5.6.7 the architecture is not included in the
            # pkg name for the OSX release as it is considered a "univeral binary"
            archive_name_template = "mathsat-{version}-{os}.{ext}"
            format["os"] = "osx"

        archive_name = archive_name_template.format(**format)

        native_link = "http://mathsat.fbk.eu/download.php?file={archive_name}"

        SolverInstaller.__init__(self, install_dir=install_dir,
                                 bindings_dir=bindings_dir,
                                 solver_version=solver_version,
                                 archive_name=archive_name,
                                 native_link = native_link,
                                 mirror_link=mirror_link)

        self.python_bindings_dir = os.path.join(self.extract_path, "python")


    def compile(self):
        if self.os_name == "windows":
            libdir = os.path.join(self.python_bindings_dir, "../lib")
            incdir = os.path.join(self.python_bindings_dir, "../include")
            gmp_h_url = "https://github.com/mikand/tamer-windows-deps/raw/master/gmp/include/gmp.h"
            mpir_dll_url = "https://github.com/Legrandin/mpir-windows-builds/blob/master/mpir-2.6.0_VS2015_%s/mpir.dll?raw=true" % self.bits
            mpir_lib_url = "https://github.com/Legrandin/mpir-windows-builds/blob/master/mpir-2.6.0_VS2015_%s/mpir.lib?raw=true" % self.bits
            setup_py_win_url = "https://github.com/pysmt/solvers_patches/raw/master/mathsat/setup-win.py"

            SolverInstaller.do_download(gmp_h_url, os.path.join(incdir, "gmp.h"))
            SolverInstaller.do_download(mpir_dll_url, os.path.join(libdir, "mpir.dll"))
            SolverInstaller.do_download(mpir_lib_url, os.path.join(libdir, "mpir.lib"))

            # Overwrite setup.py with the patched version
            setup_py = os.path.join(self.python_bindings_dir, "setup.py")
            SolverInstaller.mv(setup_py, setup_py + ".original")
            SolverInstaller.do_download(setup_py_win_url, setup_py)

        # Run setup.py to compile the bindings
        if self.os_name in {"windows", "darwin"}:
            SolverInstaller.run_python("./setup.py build_ext", self.python_bindings_dir)
        else:
            # NB: -R adds --rpath=$ORIGIN to link step, which makes shared library object
            # searched for in the extension's directory (no need for LD_LIBRARY_PATH)
            # (note: this is the default behavior for DLL discovery on Windows)
            SolverInstaller.run_python("./setup.py build_ext -R $ORIGIN", self.python_bindings_dir)


    def move(self):
        pdir = self.python_bindings_dir
        bdir = os.path.join(pdir, "build")
        sodir = glob.glob(bdir + "/lib.*")[0]
        libdir = os.path.join(self.python_bindings_dir, "../lib")

        # First, we need the SWIG-generated wrapper
        for f in os.listdir(sodir):
            if f.endswith(".so") or f.endswith(".pyd"):
                SolverInstaller.mv(os.path.join(sodir, f), self.bindings_dir)
        SolverInstaller.mv(os.path.join(pdir, "mathsat.py"), self.bindings_dir)

        # Since MathSAT 5.5.0 we also need the SO/DLL/DYLIB of mathsat in the PATH
        # Under Windows, we also need the DLLs of MPIR in the PATH
        for f in os.listdir(libdir):
            if f.endswith(".so") or f.endswith(".dll") or f.endswith(".dylib"):
                SolverInstaller.mv(os.path.join(libdir, f), self.bindings_dir)

        # Fix issue in MathSAT 5.6.10 linking to incorrect directory on OSX
        if self.os_name == "darwin":
            soname = glob.glob(self.bindings_dir + "/_mathsat*.so")[0]
            old_path = "/Users/alb/src/release/build/libmathsat.dylib"
            new_path = "%s/libmathsat.dylib" % self.bindings_dir
            SolverInstaller.run("install_name_tool -change %s %s %s" %
                                (old_path, new_path, soname))


    def get_installed_version(self):
        return self.get_installed_version_script(self.bindings_dir, "msat")



#
# This file is part of pySMT.
#
#   Copyright 2014 Andrea Micheli and Marco Gario
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
from io import StringIO

import pysmt.logics as logics
import pysmt.smtlib.commands as smtcmd
from pysmt.shortcuts import (Real, Plus, Symbol, Equals, And, Bool, Or, Not,
                             Div, LT, LE, Int, ToReal, Iff, Exists, Times, FALSE,
                             BVLShr, BVLShl, BVAShr, BV, BVAdd, BVULT, BVMul,
                             Select, Array, Ite, String, Function, to_smtlib, ForAll)
from pysmt.shortcuts import Solver, get_env, qelim, get_model, TRUE, ExactlyOne
from pysmt.typing import REAL, BOOL, INT, BVType, FunctionType, ArrayType
from pysmt.test import (TestCase, skipIfSolverNotAvailable, skipIfNoSolverForLogic,
                        skipIfNoQEForLogic)
from pysmt.test import main
from pysmt.exceptions import (ConvertExpressionError, PysmtValueError,
                              PysmtTypeError, InternalSolverError)
from pysmt.test.examples import get_example_formulae
from pysmt.environment import Environment
from pysmt.rewritings import cnf_as_set
from pysmt.smtlib.parser import SmtLibParser
from pysmt.smtlib.commands import DECLARE_FUN
from pysmt.smtlib.script import SmtLibCommand
from pysmt.logics import get_closer_smtlib_logic
from pysmt.constants import Fraction


class TestRegressions(TestCase):

    @skipIfSolverNotAvailable("msat")
    @skipIfSolverNotAvailable("z3")
    def test_plus_converts_correctly_n_ary_functions(self):
        """Handling of Plus n-ary functionality.

        Only the first two elements were translated to the solver
        """

        a = Symbol("a", REAL)
        b = Symbol("b", REAL)
        c = Symbol("c", REAL)

        p1 = Plus(a, Real((1,6)), b,c,)
        p2 = Plus(a, b, c, Real((1,6)))


        self.assertValid(Equals(p1, p2))
        self.assertValid(Equals(p1, p2), solver_name='z3')
        self.assertValid(Equals(p1, p2), solver_name='msat')

    def test_substitute_memoization(self):
        a = Symbol("A", BOOL)
        b = Symbol("B", BOOL)

        f = And(a, b)
        g = f.substitute({a:Bool(True)})
        h = f.substitute({a:Bool(False)})

        self.assertNotEqual(h, g)

    @skipIfSolverNotAvailable("msat")
    def test_msat_bool_back_conversion(self):
        f = Symbol("A")
        with Solver(name='msat') as solver:
            solver.solve()
            val = solver.get_value(Symbol("A"))
            self.assertTrue(val.is_bool_constant())

    @skipIfSolverNotAvailable("msat")
    @skipIfSolverNotAvailable("z3")
    def test_conversion_of_fractions_in_z3(self):
        self.assertValid(Equals(Real(Fraction(1,9)),
                                Div(Real(1), Real(9))),
                         solver_name="msat")
        self.assertValid(Equals(Real(Fraction(1,9)),
                                Div(Real(1), Real(9))),
                         solver_name="z3")

    def test_simplifying_int_plus_changes_type_of_expression(self):
        varA = Symbol("At", INT)
        varB = Symbol("Bt", INT)
        get_type = get_env().stc.get_type
        f = Plus(varB, Int(1))
        old_type = get_type(f)
        f = f.simplify()
        new_type = get_type(f)
        self.assertEqual(new_type, old_type)

    @skipIfNoSolverForLogic(logics.QF_UFLIRA)
    def test_nary_operators_in_solver_converter(self):
        """Conversion of n-ary operators was not handled correctly by converters."""
        x = Symbol("x")
        r = Symbol("p", REAL)
        f_and_one = And(x)
        f_or_one = Or(x)
        f_plus_one = LT(Plus(r), Real(0))

        ten_x = [x,x,x,x,x,x,x,x,x,x]
        f_and_many = And(ten_x)
        f_or_many = Or(ten_x)
        f_plus_many = LT(Plus(r,r,r,r,r,r,r,r,r,r,r), Real(0))

        for name in get_env().factory.all_solvers(logic=logics.QF_BOOL):
            self.assertSat(f_and_one, solver_name=name)
            self.assertSat(f_or_one, solver_name=name)
            self.assertSat(f_and_many, solver_name=name)
            self.assertSat(f_or_many, solver_name=name)

        for name in get_env().factory.all_solvers(logic=logics.QF_UFLIRA):
            self.assertSat(f_plus_one, solver_name=name)
            self.assertSat(f_plus_many, solver_name=name)

    def test_dependencies_not_includes_toreal(self):
        p = Symbol("p", INT)
        r = ToReal(p)
        deps = r.get_free_variables()

        self.assertIn(p, deps)
        self.assertNotIn(r, deps)

    def test_infix_notation_wrong_le(self):
        p = Symbol("p", INT)
        get_env().enable_infix_notation = True
        self.assertEqual(LE(p, Int(2)), p <= Int(2))


    def test_multiple_declaration_w_same_functiontype(self):
        ft1 = FunctionType(REAL, [REAL])
        ft2 = FunctionType(REAL, [REAL])

        f1 = Symbol("f1", ft1)
        # The following raises an exception if not (ft1 == ft2)
        # since the same symbol has already been defined with
        # a "different" type.
        f1 = Symbol("f1", ft2)

    @skipIfSolverNotAvailable("z3")
    def test_z3_iff(self):
        z3 = Solver(name="z3")
        conv = z3.converter

        x, y = Symbol("x"), Symbol("y")
        term = conv.convert(Iff(x, y))
        back = conv.back(term)
        self.assertEqual(Iff(x, y), back)

    @skipIfSolverNotAvailable("msat")
    def test_msat_iff(self):
        msat = Solver(name="msat")
        conv = msat.converter

        x, y = Symbol("x"), Symbol("y")
        term = conv.convert(Iff(x, y))
        back = conv.back(term)
        # Mathsat can reorder variables...
        self.assertTrue(Iff(x, y) == back or Iff(y, x) == back)


    def test_multiple_exit(self):
        for sname in get_env().factory.all_solvers():
            # Multiple exits should be ignored
            s = Solver(name=sname)
            s.exit()
            s.exit()
            self.assertTrue(True)

    @skipIfNoQEForLogic(logics.LIA)
    def test_lia_qe_requiring_modulus(self):
        x = Symbol("x", INT)
        y = Symbol("y", INT)
        f = Exists([x], Equals(y, Times(x, Int(2))))
        with self.assertRaises(ConvertExpressionError):
            qelim(f)

        try:
            qelim(f)
        except ConvertExpressionError as ex:
            # The modulus operator must be there
            self.assertTrue("%2" in str(ex.expression) or \
                            "int_mod_congr" in str(ex.expression))

    @skipIfSolverNotAvailable("msat")
    def test_msat_partial_model(self):
        msat = Solver(name="msat")
        x, y = Symbol("x"), Symbol("y")
        msat.add_assertion(x)
        c = msat.solve()
        self.assertTrue(c)

        model = msat.get_model()
        self.assertNotIn(y, model)
        self.assertIn(x, model)
        msat.exit()

    @skipIfSolverNotAvailable("z3")
    def test_z3_model_iteration(self):
        x, y = Symbol("x"), Symbol("y")
        m = get_model(And(x, y), solver_name="z3")
        self.assertIsNotNone(m)

        for _, v in m:
            self.assertEqual(v, TRUE())

    def test_exactlyone_w_generator(self):
        x, y = Symbol("x"), Symbol("y")

        elems = [x,y]
        f1 = ExactlyOne(elems)
        f2 = ExactlyOne(e for e in elems)

        self.assertEqual(f1, f2)

    def test_determinism(self):
        def get_set(env):
            mgr = env.formula_manager
            r = set(mgr.Symbol("x%d" % i) for i in range(1000))
            for (f, _, _, _) in get_example_formulae(env):
                r |= set([f])
            return r

        # As first thing on the environment we build the set of formulae
        l1 = list(get_set(get_env()))

        # We try this ten times...
        for _ in range(10):
            # Do something to screw up memory layout...
            for y in (Symbol("y%d" % i) for i in range(1000)):
                self.assertIsNotNone(y)

            with Environment() as new_env:
                # As first thing on the environment we build the set of formulae
                l_test = list(get_set(new_env))

                # The ordering of the sets should be the same...
                for i,f in enumerate(l1):
                    nf = new_env.formula_manager.normalize(f)
                    self.assertEqual(nf, l_test[i])

    def test_is_one(self):
        self.assertTrue(Int(1).is_one())
        self.assertTrue(Real(1).is_one())
        self.assertTrue(Int(0).is_zero())
        self.assertTrue(Real(0).is_zero())

    def test_cnf_as_set(self):
        r = cnf_as_set(Symbol("x"))
        self.assertTrue(type(r) == frozenset)

    def test_substitute_to_real(self):
        p = Symbol("p", INT)
        f = LT(ToReal(p), Real(0))

        new_f = f.substitute({p: Real(1)}).simplify()
        self.assertEqual(new_f, Bool(False))

    def test_empty_string_symbol(self):
        with self.assertRaises(PysmtValueError):
            Symbol("")

    def test_smtlib_info_quoting(self):
        cmd = SmtLibCommand(smtcmd.SET_INFO, [":source", "This\nis\nmultiline!"])
        output = cmd.serialize_to_string()
        self.assertEqual(output, "(set-info :source |This\nis\nmultiline!|)")

    def test_parse_define_fun(self):
        smtlib_input = "(declare-fun z () Bool)"\
                       "(define-fun .def_1 ((z Bool)) Bool (and z z))"
        parser = SmtLibParser()
        buffer_ = StringIO(smtlib_input)
        parser.get_script(buffer_)

    def test_parse_define_fun_bind(self):
        smtlib_input = "(declare-fun y () Bool)"\
                       "(define-fun .def_1 ((z Bool)) Bool (and z z))"
        parser = SmtLibParser()
        buffer_ = StringIO(smtlib_input)
        parser.get_script(buffer_)

    def test_parse_bvx_var(self):
        """bvX is a valid identifier."""
        smtlib_input = """
        (declare-fun bv1 () (_ BitVec 8))
        (assert (bvult (_ bv0 8) (bvmul (bvadd bv1 (_ bv1 8)) (_ bv5 8))))
        (check-sat)"""
        parser = SmtLibParser()
        buffer_ = StringIO(smtlib_input)
        script = parser.get_script(buffer_)
        # Check Parsed result
        iscript = iter(script)
        cmd = next(iscript)
        self.assertEqual(cmd.name, DECLARE_FUN)
        bv1 = cmd.args[0]
        self.assertEqual(bv1.symbol_type().width, 8)
        cmd = next(iscript)
        parsed_f = cmd.args[0]
        target_f = BVULT(BV(0, 8),
                         BVMul(BVAdd(bv1, BV(1, 8)), BV(5, 8)))
        self.assertEqual(parsed_f, target_f)


    def test_simplify_times(self):
        a,b = Real(5), Real((1,5))
        f = Times(a,b).simplify()
        self.assertEqual(f.constant_value(), 1)

    @skipIfSolverNotAvailable("yices")
    def test_yices_push(self):
        with Solver(name="yices") as solver:
            solver.add_assertion(FALSE())
            res = solver.solve()
            self.assertFalse(res)
            solver.push()
            solver.add_assertion(TRUE())
            res = solver.solve()
            self.assertFalse(res)
            solver.pop()

    def test_qf_bool_smt2(self):
        # QF_BOOL does not exist in SMT-LIB
        # This test is to enforce the consistent choice of QF_UF
        close_l = get_closer_smtlib_logic(logics.QF_BOOL)
        self.assertEqual(close_l, logics.QF_UF)
        # For BOOL we use LRA
        close_l = get_closer_smtlib_logic(logics.BOOL)
        self.assertEqual(close_l, logics.LRA)

    def test_exactly_one_unpacking(self):
        s1,s2 = Symbol("x"), Symbol("y")
        f1 = ExactlyOne((s for s in [s1,s2]))
        f2 = ExactlyOne([s1,s2])
        f3 = ExactlyOne(s1,s2)

        self.assertEqual(f1,f2)
        self.assertEqual(f2,f3)

    @skipIfSolverNotAvailable("btor")
    def test_btor_bitwidth_bug_in_shift(self):
        # (384, 384, 9)
        # (x69 >> 1_384)
        s = Solver(name="btor")
        x69 = Symbol("x69", BVType(384))
        # BVLShr
        f = BVLShr(x69, BV(1, 384))
        c = s.converter.convert(f)
        self.assertIsNotNone(c)
        # BVLShl
        f = BVLShl(x69, BV(1, 384))
        c = s.converter.convert(f)
        self.assertIsNotNone(c)
        # BVAShr
        f = BVAShr(x69, BV(1, 384))
        c = s.converter.convert(f)
        self.assertIsNotNone(c)

    @skipIfSolverNotAvailable("btor")
    def test_btor_get_non_bool_value(self):
        with Solver(name="btor") as s:
            x = Symbol("x", BVType(16))
            s.add_assertion(Equals(x, BV(1, 16)))
            self.assertTrue(s.solve())
            self.assertEqual(s.get_value(Equals(x, BV(1, 16))), TRUE())
            self.assertEqual(s.get_value(BVAdd(x, BV(1, 16))), BV(2, 16))

    @skipIfSolverNotAvailable("btor")
    def test_btor_get_array_element(self):
        with Solver(name="btor") as s:
            x = Symbol("a", ArrayType(BVType(16), BVType(16)))
            s.add_assertion(Equals(Select(x, BV(1, 16)), BV(1, 16)))
            s.add_assertion(Equals(Select(x, BV(2, 16)), BV(3, 16)))
            self.assertTrue(s.solve())
            self.assertEqual(s.get_value(Select(x, BV(1, 16))), BV(1, 16))
            self.assertIsNotNone(s.get_value(x))


    def test_smtlib_define_fun_serialization(self):
        smtlib_input = "(define-fun init ((x Bool)) Bool (and x (and x (and x (and x (and x (and x x)))))))"
        parser = SmtLibParser()
        buffer_ = StringIO(smtlib_input)
        s = parser.get_script(buffer_)
        for c in s:
            res = c.serialize_to_string(daggify=False)
        self.assertEqual(res.replace('__x0', 'x'), smtlib_input)

    @skipIfSolverNotAvailable("z3")
    def test_z3_nary_back(self):
        from z3 import Tactic
        r = Symbol("r", REAL)
        s = Symbol("s", REAL)
        t = Symbol("t", REAL)
        f = Equals(Times(r,s,t), Real(0))

        with Solver(name="z3") as solver:
            z3_f = solver.converter.convert(f)
            z3_f = Tactic('simplify', solver.z3.ctx)(z3_f).as_expr()
            fp = solver.converter.back(z3_f)
            self.assertValid(Iff(f, fp), (f, fp))

    def test_array_initialization_printing(self):
        self.assertEqual(str(Array(INT, Int(0), {Int(1):Int(2)})), "Array{Int, Int}(0)[1 := 2]")

    @skipIfSolverNotAvailable("btor")
    def test_boolector_assumptions(self):
        with Solver(name='btor') as solver:
            x = Symbol('x')
            y = Symbol('y')
            solver.add_assertion(Or(x, y))
            solver.solve([Not(x), Not(y)])
            btor_notx = solver.converter.convert(Not(x))
            btor_noty = solver.converter.convert(Not(y))
            self.assertEqual(solver.btor.Failed(btor_notx, btor_noty),
                             [True, True])

    def test_parse_declare_const(self):
        smtlib_input = """
        (declare-const s Int)
        (check-sat)"""
        parser = SmtLibParser()
        buffer_ = StringIO(smtlib_input)
        script = parser.get_script(buffer_)
        self.assertIsNotNone(script)

    @skipIfSolverNotAvailable("z3")
    def test_z3_conversion_ite(self):
        with Solver(name='z3') as solver:
            x = Symbol('x')
            y = Symbol('y')
            f = Ite(x, y, FALSE())
            solver.add_assertion(f)
            self.assertTrue(solver.solve())

    def test_parse_exception(self):
        from pysmt.exceptions import PysmtSyntaxError
        smtlib_input = "(declare-const x x x Int)" +\
                       "(check-sat)"
        parser = SmtLibParser()
        buffer_ = StringIO(smtlib_input)
        try:
            parser.get_script(buffer_)
            self.assertFalse(True)
        except PysmtSyntaxError as ex:
            self.assertEqual(ex.pos_info[0], 0)
            self.assertEqual(ex.pos_info[1], 19)

    def test_parse_bvconst_width(self):
        smtlib_input = "(assert (> #x10 #x10))"
        parser = SmtLibParser()
        buffer_ = StringIO(smtlib_input)
        expr = parser.get_script(buffer_).get_last_formula()
        const = expr.args()[0]
        self.assertEqual(const.bv_width(), 8, const.bv_width())

    def test_equality_typing(self):
        x = Symbol('x', BOOL)
        y = Symbol('y', BOOL)
        with self.assertRaises(PysmtTypeError):
            Equals(x, y)
        with self.assertRaises(PysmtTypeError):
            LE(x, y)

    def test_string_constant_quote_escaping_hr_printer(self):
        x = String('a"b')
        y = String('""')
        z = String('"')
        self.assertEqual('"a""b"', str(x))
        self.assertEqual('""""""', str(y))
        self.assertEqual('""""', str(z))

    def test_string_constant_quote_escaping_smtlib_printer(self):
        x = String('a"b')
        y = String('""')
        z = String('"')
        self.assertEqual('"a""b"', x.to_smtlib())
        self.assertEqual('""""""', y.to_smtlib())
        self.assertEqual('""""', z.to_smtlib())

    def test_string_constant_quote_escaping_parsing(self):
        script = """
        (declare-const x String)
        (assert (= x "a""b"))
        (assert (= x "\"\""))
        (assert (= x "\"\"\"\""))
        """

        p = SmtLibParser()
        buffer = StringIO(script)
        s = p.get_script(buffer)
        self.assertEqual('a"b', s.commands[1].args[0].arg(1).constant_value())
        self.assertEqual('"', s.commands[2].args[0].arg(1).constant_value())
        self.assertEqual('""', s.commands[3].args[0].arg(1).constant_value())

    def test_pysmt_syntax_error(self):
        from pysmt.exceptions import PysmtSyntaxError
        try:
            raise PysmtSyntaxError("'define-fun' expected", (5,5))
        except PysmtSyntaxError as ex:
            self.assertEqual(str(ex), "Line 5, Col 5: 'define-fun' expected")

    def test_function_smtlib_print(self):
        f_t = FunctionType(BOOL, [BOOL])
        f0 = Symbol('f 0', f_t)
        f0_of_false = Function(f0, [Bool(False)])
        s = to_smtlib(f0_of_false, False)
        self.assertEqual(s, '(|f 0| false)')

    @skipIfSolverNotAvailable("yices")
    def test_yices_bv_overflow(self):
        smt_script = '''
        (set-logic QF_BV)
        (declare-fun s0 () (_ BitVec 64))
        (define-fun s1 () (_ BitVec 64) #xFFFFFFFFFFFFFFFF)
        (define-fun s2 () Bool (= s1 s0))
        (assert s2)
        (check-sat)
        '''
        from pysmt.smtlib.parser import get_formula_strict
        f = get_formula_strict(StringIO(smt_script))
        self.assertSat(f, solver_name='yices')

    @skipIfSolverNotAvailable("yices")
    def test_yices_bv_no_overflow(self):
        smt_script = '''
        (set-logic QF_BV)
        (declare-fun s0 () (_ BitVec 64))
        (define-fun s1 () (_ BitVec 64) #x7FFFFFFFFFFFFFFF)
        (define-fun s2 () Bool (= s1 s0))
        (assert s2)
        (check-sat)
        '''
        from pysmt.smtlib.parser import get_formula_strict
        f = get_formula_strict(StringIO(smt_script))
        self.assertSat(f, solver_name='yices')

    def test_get_atoms_array_select(self):
        a = Symbol("a", ArrayType(INT, BOOL))
        x = Symbol("x", INT)
        p = Symbol("p", BOOL)

        phi = And(Iff(Select(a, x), p), Equals(x, Int(1)))

        atoms = phi.get_atoms()

        self.assertEqual(len(atoms), 3)
        self.assertIn(Select(a, x), atoms)
        self.assertIn(p, atoms)
        self.assertIn(Equals(x, Int(1)), atoms)


    @skipIfSolverNotAvailable("yices")
    def test_yices_quantifier(self):
        x = Symbol('x', REAL)
        f = ForAll([x], LE(x, Real(0)))
        with self.assertRaises(InternalSolverError):
            with Solver(name='yices') as s:
                s.add_assertion(f)
                self.assertFalse(s.solve())


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import math
import argparse

from pysmt.shortcuts import Int, FreshSymbol, Solver, Equals
from pysmt.shortcuts import Or, get_env, BV, AllDifferent
from pysmt import typing
from pysmt.logics import QF_LIA, QF_BV


class Sudoku(object):
    """A sudoku problem representation and solving"""

    def __init__(self, size=3, solver_name=None, use_bv=None):
        self.size = size

        if use_bv is None:
            # If BV is unspecified, check to see if a BV solver is
            # available, otherwise default to INT
            all_bv_solvers = get_env().factory.all_solvers(QF_BV)
            use_bv = (len(all_bv_solvers) > 0)
        self.use_bv = use_bv

        # Logic selection
        logic = QF_LIA
        if use_bv:
            logic = QF_BV
            # Compute the minimum BV size needed for this problem
            self.bv_size = int(math.floor(math.log(self.size**2, 2))) + 1

        # Build a solver initialized with the proper assertions and
        # the matrix of SMT-variables representing each cell.
        self.solver, self.var_table = \
              self.get_prepared_solver(logic, solver_name)


    def get_type(self):
        """Returns the type of variables: either INT or BV depending on the
        use_bv flag of the constructor
        """
        if self.use_bv:
            return typing.BVType(self.bv_size)
        else:
            return typing.INT

    def const(self, value):
        """Create a constant with the given value of the correct type"""
        if self.use_bv:
            return BV(value, self.bv_size)
        else:
            return Int(value)

    def get_prepared_solver(self, logic, solver_name=None):
        """Returns a solver initialized with the sudoku constraints and a
        matrix of SMT variables, each representing a cell of the game.
        """
        sq_size = self.size**2
        ty = self.get_type()
        var_table = [[FreshSymbol(ty) for _ in range(sq_size)]
                          for _ in range(sq_size)]

        solver = Solver(logic=logic, name=solver_name)
        # Sudoku constraints
        # all variables are positive and lower or equal to than sq_size
        for row in var_table:
            for var in row:
                solver.add_assertion(Or([Equals(var, self.const(i))
                                         for i in range(1, sq_size + 1)]))

        # each row and each column contains all different numbers
        for i in range(sq_size):
            solver.add_assertion(AllDifferent(var_table[i]))
            solver.add_assertion(AllDifferent([x[i] for x in var_table]))

        # each square contains all different numbers
        for sx in range(self.size):
            for sy in range(self.size):
                square = [var_table[i + sx * self.size][j + sy * self.size]
                          for i in range(self.size) for j in range(self.size)]
                solver.add_assertion(AllDifferent(square))

        return solver, var_table

    def solve(self, constraints=None):
        """Solve a problem with the given constraints.

        constraints is a matrix of integer numbers, -1 indicates an
        empty cell.

        The function returns a completed matrix of integer numbers or
        None if the problem is unsolvable

        """
        self.solver.push()

        if constraints is not None:
            # User constraints
            for i, row in enumerate(constraints):
                for j, val in enumerate(row):
                    if val > 0:
                        self.solver.add_assertion(Equals(self.var_table[i][j],
                                                         self.const(val)))

        res = self.solver.solve()

        if res:
            result = []
            for r, lst in enumerate(self.var_table):
                r = []
                for var in lst:
                    val = self.solver.get_py_value(var)
                    r.append(val)
                result.append(r)
            self.solver.pop()
            return result
        else:
            self.solver.pop()
            return None



def read_constraints(fname):
    """reads the constraints from file
    (see problems/ folder for examples)
    """
    def reader(fname):
        with open(fname, "r") as f:
            for line in f:
                r = line.split()
                for x in r:
                    yield x

    r = reader(fname)
    size = int(next(r))
    constraints = []
    for _ in range(size**2):
        row = []
        for _ in range(size**2):
            w = next(r)
            if w != "X":
                row.append(int(w))
            else:
                row.append(-1)
        constraints.append(row)
    return constraints


def main():
    env = get_env()

    parser = argparse.ArgumentParser(description="Command-line interface " \
                                     "for solving sudoku problems")

    parser.add_argument('--size', '-n', metavar='number', type=int,
                        help='The sudoku base size', default=3)

    parser.add_argument('--solver', '-s', metavar='name', type=str,
                        choices=['auto'] + list(env.factory.all_solvers().keys()),
                        default='auto',
                        help='The solver to use (default: auto)')

    parser.add_argument('--bv', '-b', action="store_true",
                        help='Force the use of bit-vectors instead of integers')

    parser.add_argument('--lia', '-i', action="store_true",
                        help='Force use LIA instead of bit-vectors')

    parser.add_argument('--problem', '-p', metavar='filename', type=str,
                        help='The sudoku problem')

    args = parser.parse_args()

    solver = None if args.solver == "auto" else args.solver
    use_bv = True if args.bv else (False if args.lia else None)
    sudoku = Sudoku(size=args.size,
                    solver_name=solver,
                    use_bv=use_bv)

    constraints = None
    if args.problem:
        constraints = read_constraints(args.problem)
        res = sudoku.solve(constraints)
        if res is None:
            print("No solution exists")
        else:
            for row in res:
                print("\t".join(str(x) for x in row))
    else:
        print("No problem specified! Either use the gui.py script or pass" \
            " a problem with --problem flag. Example problems are available" \
            " in the problems/ folder.")



if __name__ == "__main__":
    main()



#
# This file is part of pySMT.
#
#   Copyright 2014 Andrea Micheli and Marco Gario
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#

class Interpolator(object):

    def __init__(self):
        self._destroyed = False

    def binary_interpolant(self, a, b):
        """Returns a binary interpolant for the pair (a, b), if And(a, b) is
        unsatisfiable, or None if And(a, b) is satisfiable.

        """
        raise NotImplementedError

    def sequence_interpolant(self, formulas):
        """Returns a sequence interpolant for the conjunction of formulas, or
        None if the problem is satisfiable.

        """
        raise NotImplementedError

    def __enter__(self):
        """Manage entering a Context (i.e., with statement)"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Manage exiting from Context (i.e., with statement)

        The default behaviour is to explicitly destroy the interpolator to
        free the associated resources.

        """
        self.exit()

    def exit(self):
        """Destroys the solver and closes associated resources."""
        if not self._destroyed:
            self._exit()
            self._destroyed = True

    def _exit(self):
        """Destroys the solver and closes associated resources."""
        raise NotImplementedError



#
# This file is part of pySMT.
#
#   Copyright 2014 Andrea Micheli and Marco Gario
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
import pysmt.operators as op
from pysmt.shortcuts import FreshSymbol, Symbol, Int, Bool, ForAll
from pysmt.shortcuts import And, Or, Iff, Not, Function, Real
from pysmt.shortcuts import LT, GT, Plus, Minus, Equals
from pysmt.shortcuts import get_env, substitute, TRUE
from pysmt.typing import INT, BOOL, REAL, FunctionType
from pysmt.walkers import TreeWalker, DagWalker, IdentityDagWalker
from pysmt.test import TestCase, main
from pysmt.formula import FormulaManager
from pysmt.test.examples import get_example_formulae
from pysmt.exceptions import UnsupportedOperatorError, PysmtTypeError
from pysmt.substituter import MSSubstituter, MGSubstituter


class TestWalkers(TestCase):

    def test_subst(self):
        varA = Symbol("At", INT)
        varB = Symbol("Bt", INT)

        f = And(LT(varA, Plus(varB, Int(1))),
                GT(varA, Minus(varB, Int(1))))
        g = Equals(varA, varB)
        h = Iff(f, g)

        res = substitute(h, subs={varA:varB})

        self.assertEqual(res, h.substitute({varA:varB}))

        res = substitute(h, subs={varA:Int(1)})
        self.assertEqual(res, h.substitute({varA:Int(1)}))

    def test_substituter_conditions(self):
        x = Symbol("x")
        y = Symbol("y")
        and_x_x = And(x, x)
        ftype = FunctionType(BOOL, [BOOL])
        f = Symbol("f", ftype)

        # 1. All arguments must be terms
        args_good = {x:y}
        args_bad = {x:f}

        substitute(and_x_x, args_good)
        with self.assertRaisesRegex(PysmtTypeError, " substitutions"):
            substitute(and_x_x, args_bad)

        # 2. All arguments belong to the manager of the substituter.
        new_mgr = FormulaManager(get_env())
        new_x = new_mgr.Symbol("x")
        self.assertNotEqual(x, new_x)
        args_1 = {x: new_x}
        args_2 = {new_x: new_x}

        with self.assertRaisesRegex(PysmtTypeError, "Formula Manager" ):
            substitute(and_x_x, args_1)

        with self.assertRaisesRegex(PysmtTypeError, "Formula Manager."):
            substitute(and_x_x, args_2)

        with self.assertRaisesRegex(PysmtTypeError, "substitute()"):
            substitute(f, {x:x})

    def test_undefined_node(self):
        varA = Symbol("At", INT)

        dag_walker = DagWalker()
        with self.assertRaises(UnsupportedOperatorError):
            dag_walker.walk(varA)

        tree_walker = TreeWalker()
        with self.assertRaises(UnsupportedOperatorError):
            tree_walker.walk(varA)

    def test_walker_new_operators_complete(self):
        walkerA = IdentityDagWalker(env=self.env)
        idx = op.new_node_type(node_str="fancy_new_node")
        walkerB = IdentityDagWalker(env=self.env)
        with self.assertRaises(KeyError):
            walkerA.functions[idx]
        self.assertEqual(walkerB.functions[idx], walkerB.walk_error)

        # Use a mixin to handle the node type
        class FancyNewNodeWalkerMixin(object):
            def walk_fancy_new_node(self, args, **kwargs):
                raise UnsupportedOperatorError

        class IdentityDagWalker2(IdentityDagWalker, FancyNewNodeWalkerMixin):
            pass
        walkerC = IdentityDagWalker2(env=self.env)
        self.assertEqual(walkerC.functions[idx],
                         walkerC.walk_fancy_new_node)

    def test_identity_walker_simple(self):

        def walk_and_to_or(formula, args, **kwargs):
            return Or(args)

        def walk_or_to_and(formula, args, **kwargs):
            return And(args)

        walker = IdentityDagWalker(env=get_env())

        walker.set_function(walk_and_to_or, op.AND)
        walker.set_function(walk_or_to_and, op.OR)

        x, y, z = Symbol('x'), Symbol('y'), Symbol('z')

        cnf = And(Or(x,y,z), Or(z, Not(y)))
        fake_dnf = Or(And(x,y,z), And(z, Not(y)))
        result = walker.walk(cnf)
        self.assertEqual(result, fake_dnf)

        alternation = Or(cnf, Not(cnf))
        expected = And(fake_dnf, Not(fake_dnf))
        result = walker.walk(alternation)
        self.assertEqual(result, expected)

    def test_identity_dag_walker(self):
        idw = IdentityDagWalker()
        for (f, _, _, _) in get_example_formulae():
            rebuilt = idw.walk(f)
            self.assertTrue(rebuilt == f, "Rebuilt formula is not identical")

    def test_substitution_on_quantifiers(self):
        x, y = FreshSymbol(), FreshSymbol()

        # y /\ Forall x. x /\ y.
        f = And(y, ForAll([x], And(x, y)))

        # Symbols within the quantified formula are not free symbols
        # and should not be substituted.
        subs = {y: TRUE()}
        f_subs = substitute(f, subs).simplify()
        self.assertEqual(f_subs, ForAll([x], x))

        subs = {x: TRUE()}
        f_subs = substitute(f, subs).simplify()
        self.assertEqual(f_subs, f)

    def test_substitution_complex(self):
        x, y = Symbol("x", REAL), Symbol("y", REAL)
        # y = 0 /\ (Forall x. x > 3 /\ y < 2)
        f = And(Equals(y, Real(0)),
                ForAll([x], And(GT(x, Real(3)), LT(y, Real(2)))))

        subs = {y: Real(0),
                ForAll([x], And(GT(x, Real(3)), LT(y, Real(2)))): TRUE()}
        f_subs = substitute(f, subs).simplify()
        if self.env.SubstituterClass == MGSubstituter:
            self.assertEqual(f_subs, TRUE())
        else:
            # In the MSS the y=0 substitution is performed first,
            # therefore, the overall quantified expression does not
            # match the one defined in the substitution map.
            # See test_substitution_complex_mss for a positive example.
            self.assertEqual(f_subs, ForAll([x], GT(x, Real(3))))

    def test_substitution_complex_mss(self):
        x, y = FreshSymbol(REAL), FreshSymbol(REAL)
        # y = 0 /\ (Forall x. x > 3 /\ y < 2)
        f = And(Equals(y, Real(0)),
                ForAll([x], And(GT(x, Real(3)), LT(y, Real(2)))))
        subs = {y: Real(0),
                ForAll([x], And(GT(x, Real(3)), LT(Real(0), Real(2)))): TRUE()}
        f_subs = MSSubstituter(env=self.env).substitute(f, subs).simplify()
        self.assertEqual(f_subs, TRUE())

    def test_substitution_term(self):
        x, y = FreshSymbol(REAL), FreshSymbol(REAL)

        # y = 0 /\ Forall x. x > 3
        f = And(Equals(y, Real(0)), ForAll([x], GT(x, Real(3))))

        subs = {GT(x, Real(3)): TRUE()}
        f_subs = substitute(f, subs)
        # Since 'x' is quantified, we cannot replace the term
        # therefore the substitution does not yield any result.
        self.assertEqual(f_subs, f)

    def test_substitution_on_functions(self):
        i, r = FreshSymbol(INT), FreshSymbol(REAL)
        f = Symbol("f", FunctionType(BOOL, [INT, REAL]))

        phi = Function(f, [Plus(i, Int(1)), Minus(r, Real(2))])

        phi_sub = substitute(phi, {i: Int(0)}).simplify()
        self.assertEqual(phi_sub, Function(f, [Int(1), Minus(r, Real(2))]))

        phi_sub = substitute(phi, {r: Real(0)}).simplify()
        self.assertEqual(phi_sub, Function(f, [Plus(i, Int(1)), Real(-2)]))

        phi_sub = substitute(phi, {r: Real(0), i: Int(0)}).simplify()
        self.assertEqual(phi_sub, Function(f, [Int(1), Real(-2)]))

    def test_iterative_get_free_variables(self):
        f = Symbol("x")
        for _ in range(1000):
            f = And(f, f)

        cone = f.get_free_variables()
        self.assertEqual(cone, set([Symbol("x")]))

    def test_walk_error(self):
        """All walk methods by default call walk_error."""
        from pysmt.walkers import DagWalker

        x = Symbol("x")
        w = DagWalker()
        for o in op.ALL_TYPES:
            with self.assertRaises(UnsupportedOperatorError):
                w.functions[o](x)

    def test_walker_super(self):
        from pysmt.walkers import DagWalker
        class WalkA(DagWalker):
            def __init__(self):
                DagWalker.__init__(self)
            def walk_symbol(self, formula, args, **kwargs):
                return "A" + str(formula)

        class WalkB(WalkA):
            def __init__(self):
                WalkA.__init__(self)
            def walk_symbol(self, formula, args, **kwargs):
                s = WalkA.super(self, formula, args, **kwargs)
                return "B" + str(s)

        class WalkC(WalkB):
            def __init__(self):
                WalkB.__init__(self)
            def walk_symbol(self, formula, args, **kwargs):
                s = WalkB.walk_symbol(self, formula, args, **kwargs)
                return "C" + str(s)

        wc = WalkC()
        res = wc.walk(Symbol("x"))
        self.assertEqual(res, "CBAx")

    def test_substituter_instance(self):
        from pysmt.substituter import Substituter
        with self.assertRaises(NotImplementedError):
            Substituter(env=self.env)

if __name__ == '__main__':
    main()



#
# This file is part of pySMT.
#
#   Copyright 2014 Andrea Micheli and Marco Gario
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
from pysmt.shortcuts import Solver, BVAnd, BVOr, BVXor, BVConcat, BVULT, BVUGT, \
    BVULE, BVUGE, BVAdd, BVSub, BVMul, BVUDiv, BVURem, BVLShl, BVLShr, BVNot, \
    BVNeg, BVZExt, BVSExt, BVRor, BVRol, BV, BVExtract, BVSLT, BVSLE, BVComp, \
    BVSDiv, BVSRem, BVAShr, EqualsOrIff, BVZero, BVOne, Symbol, Bool
from pysmt.typing import BVType
from pysmt.test import TestCase, skipIfSolverNotAvailable, main
from pysmt.logics import QF_BV


class TestBvSimplification(TestCase):

    def setUp(self):
        # For the preparation of assertValid()
        super(TestBvSimplification, self).setUp()

        self.width = 4
        self.bin_operators = [BVAnd, BVOr, BVXor, BVConcat, BVULT, BVUGT, BVULE,
                              BVUGE, BVAdd, BVSub, BVMul, BVUDiv, BVURem, BVLShl,
                              BVLShr, BVSLT, BVSLE, BVComp, BVSDiv, BVSRem,
                              BVAShr]
        self.unary_operators = [BVNot, BVNeg]
        self.special_operators = [BVZExt, BVSExt, BVRor, BVRol]
        self.solver = None
        self.solver_simplify = None

    def all_bv_numbers(self, width=None):
        if width is None:
            width = self.width
        for x in range(2 ** width):
            yield BV(x, width)

    @skipIfSolverNotAvailable("msat")
    def test_msat(self):
        self.solver = Solver(name="msat")
        conv = self.solver.converter
        self.solver_simplify = lambda x: conv.back(conv.convert(x))
        # Run the tests
        self.all_unary()
        self.all_binary()
        self.all_special()
        self.extract()
        self.concat_different_length()

    def check(self, f):
        simp_f = f.simplify()
        solver_f = self.solver_simplify(f)
        self.assertEqual(solver_f, simp_f)

    def all_binary(self):
        for l in self.all_bv_numbers():
            for r in self.all_bv_numbers():
                for op in self.bin_operators:
                    try:
                        self.check(op(l, r))
                    except AssertionError:
                        if op in [BVUDiv, BVSDiv, BVSRem, BVURem] and r == BV(0, r.bv_width()):
                            print("Warning: Division/Reminder value mismatch. (Not an error because division by 0)")
                            print(l,op,r)
                        else:
                            raise

    def all_unary(self):
        for l in self.all_bv_numbers():
            for op in self.unary_operators:
                self.check(op(l))

    def extract(self):
        for l in self.all_bv_numbers():
            for s in range(0, self.width):
                for e in range(s, self.width):
                    self.check(BVExtract(l, start=s, end=e))

    def concat_different_length(self):
        for l in self.all_bv_numbers():
            for w in [1, self.width-1, self.width+1]:
                for r in self.all_bv_numbers(w):
                    self.check(BVConcat(l, r))

    def all_special(self):
        for l in self.all_bv_numbers():
            for n in range(0, self.width + 1):
                for op in self.special_operators:
                    self.check(op(l, n))

    def check_equal_and_valid(self, formula, expected):
        simplified = formula.simplify()

        self.assertEqual(simplified, expected)

        # Check the formula equality by a solver.
        if self.env.factory.all_solvers(QF_BV):
            self.assertValid(EqualsOrIff(simplified, formula), logic=QF_BV)

    def test_bv_0_add(self):
        x = Symbol("x", BVType(32))
        f = BVAdd(BVZero(32), x)
        self.check_equal_and_valid(f, x)

    def test_bv_add_0(self):
        x = Symbol("x", BVType(32))
        f = BVAdd(x, BVZero(32))
        self.check_equal_and_valid(f, x)

    def test_bv_add_constants(self):
        f = BVAdd(BV(10, 32), BV(12, 32))
        self.check_equal_and_valid(f, BV(22, 32))

    def test_bv_add_symbols(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVAdd(x, y)
        self.check_equal_and_valid(f, BVAdd(x, y))

    def test_bv_add_overflow(self):
        f = BVAdd(BV(2**32 - 1, 32), BV(10, 32))
        self.check_equal_and_valid(f, BV(9, 32))

    def test_bv_0_mul(self):
        x = Symbol("x", BVType(32))
        f = BVMul(BVZero(32), x)
        self.check_equal_and_valid(f, BVZero(32))

    def test_bv_1_mul(self):
        x = Symbol("x", BVType(32))
        f = BVMul(BVOne(32), x)
        self.check_equal_and_valid(f, x)

    def test_bv_mul_0(self):
        x = Symbol("x", BVType(32))
        f = BVMul(x, BVZero(32))
        self.check_equal_and_valid(f, BVZero(32))

    def test_bv_mul_1(self):
        x = Symbol("x", BVType(32))
        f = BVMul(x, BVOne(32))
        self.check_equal_and_valid(f, x)

    def test_bv_mul_constants(self):
        f = BVMul(BV(10, 32), BV(12, 32))
        self.check_equal_and_valid(f, BV(120, 32))

    def test_bv_mul_symbols(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVMul(x, y)
        self.check_equal_and_valid(f, BVMul(x, y))

    def test_bv_mul_overflow(self):
        f = BVMul(BV(2**31, 32), BV(3, 32))
        self.check_equal_and_valid(f, BV(0x80000000, 32))

    def test_bv_udiv_1(self):
        x = Symbol("x", BVType(32))
        f = BVUDiv(x, BVOne(32))
        self.check_equal_and_valid(f, x)

    def test_bv_symbol_udiv_0(self):
        x = Symbol("x", BVType(32))
        f = BVUDiv(x, BVZero(32))
        self.check_equal_and_valid(f, BV(2**32 - 1, 32))

    def test_bv_0_udiv_0(self):
        f = BVUDiv(BVZero(32), BVZero(32))
        self.check_equal_and_valid(f, BV(2**32 - 1, 32))

    def test_bv_nonzero_udiv_0(self):
        f = BVUDiv(BV(10, 32), BVZero(32))
        self.check_equal_and_valid(f, BV(2**32 - 1, 32))

    def test_bv_0_udiv_nonzero(self):
        f = BVUDiv(BVZero(32), BV(10, 32))
        self.check_equal_and_valid(f, BVZero(32))

    def test_bv_0_udiv_symbol(self):
        x = Symbol("x", BVType(32))
        f = BVUDiv(BVZero(32), x)
        self.check_equal_and_valid(f, BVUDiv(BVZero(32), x))

    def test_bv_udiv_constants(self):
        f = BVUDiv(BV(20, 32), BV(10, 32))
        self.check_equal_and_valid(f, BV(2, 32))

    def test_bv_udiv_symbols(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVUDiv(x, y)
        self.check_equal_and_valid(f, BVUDiv(x, y))

    def test_bv_urem_1(self):
        x = Symbol("x", BVType(32))
        f = BVURem(x, BVOne(32))
        self.check_equal_and_valid(f, BVZero(32))

    def test_bv_symbol_urem_0(self):
        x = Symbol("x", BVType(32))
        f = BVURem(x, BVZero(32))
        self.check_equal_and_valid(f, x)

    def test_bv_0_urem_0(self):
        f = BVURem(BVZero(32), BVZero(32))
        self.check_equal_and_valid(f, BVZero(32))

    def test_bv_nonzero_urem_0(self):
        f = BVURem(BV(10, 32), BVZero(32))
        self.check_equal_and_valid(f, BV(10, 32))

    def test_bv_0_urem_nonzero(self):
        f = BVURem(BVZero(32), BV(10, 32))
        self.check_equal_and_valid(f, BVZero(32))

    def test_bv_0_urem_symbol(self):
        x = Symbol("x", BVType(32))
        f = BVURem(BVZero(32), x)
        self.check_equal_and_valid(f, BVZero(32))

    def test_bv_urem_constants(self):
        f = BVURem(BV(20, 32), BV(11, 32))
        self.check_equal_and_valid(f, BV(9, 32))

    def test_bv_urem_symbols(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVURem(x, y)
        self.check_equal_and_valid(f, BVURem(x, y))

    def test_bv_ult_eq(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVULT(BVMul(x, y), BVMul(x, y))
        self.check_equal_and_valid(f, Bool(False))

    def test_bv_ult_zero(self):
        x = Symbol("x", BVType(32))
        f = BVULT(x, BVZero(32))
        self.check_equal_and_valid(f, Bool(False))

    def test_bv_ult_symbols(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVULT(x, y)
        self.check_equal_and_valid(f, BVULT(x, y))

    def test_bv_ult_constants(self):
        f = BVULT(BVZero(32), BVOne(32))
        self.check_equal_and_valid(f, Bool(True))

    def test_bv_ule_eq(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVULE(BVMul(x, y), BVMul(x, y))
        self.check_equal_and_valid(f, Bool(True))

    def test_bv_zero_ule(self):
        x = Symbol("x", BVType(32))
        f = BVULE(BVZero(32), x)
        self.check_equal_and_valid(f, Bool(True))

    def test_bv_ule_symbols(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVULE(x, y)
        self.check_equal_and_valid(f, BVULE(x, y))

    def test_bv_ule_constants(self):
        f = BVULE(BVZero(32), BVOne(32))
        self.check_equal_and_valid(f, Bool(True))

    def test_bv_and_zero(self):
        x = Symbol("x", BVType(32))
        f = BVAnd(x, BVZero(32))
        self.check_equal_and_valid(f, BVZero(32))

    def test_bv_zero_and(self):
        x = Symbol("x", BVType(32))
        f = BVAnd(BVZero(32), x)
        self.check_equal_and_valid(f, BVZero(32))

    def test_bv_and_all_ones(self):
        x = Symbol("x", BVType(32))
        f = BVAnd(x, BV(2**32 - 1, 32))
        self.check_equal_and_valid(f, x)

    def test_bv_all_ones_and(self):
        x = Symbol("x", BVType(32))
        f = BVAnd(BV(2**32 - 1, 32), x)
        self.check_equal_and_valid(f, x)

    def test_bv_and_symbols(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVAnd(x, y)
        self.check_equal_and_valid(f, BVAnd(x, y))

    def test_bv_and_constants(self):
        f = BVAnd(BV(0xdededede, 32), BV(0xacacacac, 32))
        self.check_equal_and_valid(f, BV(0x8c8c8c8c, 32))

    def test_bv_or_zero(self):
        x = Symbol("x", BVType(32))
        f = BVOr(x, BVZero(32))
        self.check_equal_and_valid(f, x)

    def test_bv_zero_or(self):
        x = Symbol("x", BVType(32))
        f = BVOr(BVZero(32), x)
        self.check_equal_and_valid(f, x)

    def test_bv_or_all_ones(self):
        x = Symbol("x", BVType(32))
        f = BVOr(x, BV(2**32 - 1, 32))
        self.check_equal_and_valid(f, BV(2**32 - 1, 32))

    def test_bv_all_ones_or(self):
        x = Symbol("x", BVType(32))
        f = BVOr(BV(2**32 - 1, 32), x)
        self.check_equal_and_valid(f, BV(2**32 - 1, 32))

    def test_bv_or_symbols(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVOr(x, y)
        self.check_equal_and_valid(f, BVOr(x, y))

    def test_bv_or_constants(self):
        f = BVOr(BV(0xdededede, 32), BV(0xacacacac, 32))
        self.check_equal_and_valid(f, BV(0xfefefefe, 32))

    def test_bv_sub_zero(self):
        x = Symbol("x", BVType(32))
        f = BVSub(x, BVZero(32))
        self.check_equal_and_valid(f, x)

    def test_bv_sub_eq(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVSub(BVMul(x, y), BVMul(x, y))
        self.check_equal_and_valid(f, BVZero(32))

    def test_bv_sub_symbols(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVSub(x, y)
        self.check_equal_and_valid(f, BVSub(x, y))

    def test_bv_sub_constants(self):
        f = BVSub(BV(30, 32), BV(20, 32))
        self.check_equal_and_valid(f, BV(10, 32))

    def test_bv_sub_underflow(self):
        f = BVSub(BV(1, 32), BV(2, 32))
        self.check_equal_and_valid(f, BV(0xffffffff, 32))

    def test_bv_lshl_zero(self):
        x = Symbol("x", BVType(32))
        f = BVLShl(x, BVZero(32))
        self.check_equal_and_valid(f, x)

    def test_bv_zero_lshl(self):
        x = Symbol("x", BVType(32))
        f = BVLShl(BVZero(32), x)
        self.check_equal_and_valid(f, BVZero(32))

    def test_bv_lshl_overflow(self):
        x = Symbol("x", BVType(32))
        f = BVLShl(x, BV(33, 32))
        self.check_equal_and_valid(f, BVZero(32))

    def test_bv_lshl_symbols(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVLShl(x, y)
        self.check_equal_and_valid(f, BVLShl(x, y))

    def test_bv_lshl_constants(self):
        f = BVLShl(BV(0xff, 32), BVOne(32))
        self.check_equal_and_valid(f, BV(0x1fe, 32))

    def test_bv_lshr_zero(self):
        x = Symbol("x", BVType(32))
        f = BVLShr(x, BVZero(32))
        self.check_equal_and_valid(f, x)

    def test_bv_zero_lshr(self):
        x = Symbol("x", BVType(32))
        f = BVLShr(BVZero(32), x)
        self.check_equal_and_valid(f, BVZero(32))

    def test_bv_lshr_underflow(self):
        x = Symbol("x", BVType(32))
        f = BVLShr(x, BV(33, 32))
        self.check_equal_and_valid(f, BVZero(32))

    def test_bv_lshr_symbols(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVLShr(x, y)
        self.check_equal_and_valid(f, BVLShr(x, y))

    def test_bv_lshr_constants(self):
        f = BVLShr(BV(0xff, 32), BVOne(32))
        self.check_equal_and_valid(f, BV(0x7f, 32))

    def test_bv_slt_eq(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVSLT(BVMul(x, y), BVMul(x, y))
        self.check_equal_and_valid(f, Bool(False))

    def test_bv_slt_symbols(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVSLT(x, y)
        self.check_equal_and_valid(f, BVSLT(x, y))

    def test_bv_slt_constants(self):
        f = BVSLT(BV(10, 32), BV(2**32 - 1, 32))
        self.check_equal_and_valid(f, Bool(False))

    def test_bv_sle_eq(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVSLE(BVMul(x, y), BVMul(x, y))
        self.check_equal_and_valid(f, Bool(True))

    def test_bv_sle_symbols(self):
        x, y = (Symbol(name, BVType(32)) for name in "xy")
        f = BVSLE(x, y)
        self.check_equal_and_valid(f, BVSLE(x, y))

    def test_bv_sle_constants(self):
        f = BVSLE(BV(10, 32), BV(2**32 - 1, 32))
        self.check_equal_and_valid(f, Bool(False))

if __name__ == '__main__':
    main()



# This example shows how to define a generic SMT-LIB solver, and use
# it within pySMT. The example looks for mathsat in /tmp, you can
# create a symlink there.
#
# Using this process you can experiment with any SMT-LIB 2 compliant
# solver even if it does not have python bindings, or has not been
# integrated with pySMT.
#
# Note: When using the SMT-LIB wrapper, you can only use logics
# supported by pySMT. If the version of pySMT in use does not support
# Arrays, then you cannot represent arrays.
#

# To define a Generic Solver you need to provide:
#
# - A name to associate to the solver
# - The path to the script + Optional arguments
# - The list of logics supported by the solver
#
# It is usually convenient to wrap the solver in a simple shell script.
# See examples for Z3, Mathsat and Yices in pysmt/test/smtlib/bin/*.template
#
from pysmt.logics import QF_UFLRA, QF_UFIDL, QF_LRA, QF_IDL, QF_LIA
from pysmt.shortcuts import get_env, GT, Solver, Symbol
from pysmt.typing import REAL, INT
from pysmt.exceptions import NoSolverAvailableError

name = "mathsat" # Note: The API version is called 'msat'
path = ["/tmp/mathsat"] # Path to the solver
logics = [QF_UFLRA, QF_UFIDL] # Some of the supported logics

env = get_env()

# Add the solver to the environment
env.factory.add_generic_solver(name, path, logics)

r, s = Symbol("r", REAL), Symbol("s", REAL)
p, q = Symbol("p", INT), Symbol("q", INT)

f_lra = GT(r, s)
f_idl = GT(p, q)

# PySMT takes care of recognizing that QF_LRA can be solved by a QF_UFLRA solver.
with Solver(name=name, logic=QF_LRA) as s:
    res = s.solve()
    assert res, "Was expecting '%s' to be SAT" % f_lra


with Solver(name=name, logic=QF_IDL) as s:
    s.add_assertion(f_idl)
    res = s.solve()
    assert res, "Was expecting '%s' to be SAT" % f_idl

try:
    with Solver(name=name, logic=QF_LIA) as s:
        pass
except NoSolverAvailableError:
    # If we ask for a logic that is not contained by any of the
    # supported logics an exception is thrown
    print("%s does not support QF_LIA" % name)



#
# This file is part of pySMT.
#
#   Copyright 2014 Andrea Micheli and Marco Gario
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
import os
from tempfile import NamedTemporaryFile

from pysmt.shortcuts import read_configuration, write_configuration
from pysmt.shortcuts import get_env
from pysmt.configuration import (configure_environment,
                                 write_environment_configuration)
from pysmt.environment import Environment
from pysmt.exceptions import PysmtIOError

from pysmt.test import TestCase, main
BASE_DIR = os.path.dirname(os.path.abspath(__file__))


class TestConfiguration(TestCase):

    def _get_config_path(self, config_name):
        return os.path.join(BASE_DIR, "configs", config_name)

    def _get_tmp_file(self):
        return NamedTemporaryFile(delete=False)

    def test_read(self):
        env = get_env()
        configure_environment(self._get_config_path("config1.ini"), env)
        self.assertTrue("mathsat-smt" in env.factory.all_solvers())
        self.assertTrue("z3-smt" in env.factory.all_solvers())

    def test_read_shortcut(self):
        read_configuration(self._get_config_path("config1.ini"))
        env = get_env()
        self.assertTrue("mathsat-smt" in env.factory.all_solvers())
        self.assertTrue("z3-smt" in env.factory.all_solvers())


    def test_write(self):
        env = get_env()
        configure_environment(self._get_config_path("config1.ini"), env)

        tmp = self._get_tmp_file()
        fname = tmp.name

        write_environment_configuration(fname, env)

        new_env = Environment()
        configure_environment(fname, new_env)
        try:
            os.remove(fname)
        except WindowsError:
            print("Error deleting file...")

        self.assertEqual(new_env.enable_infix_notation,
                          env.enable_infix_notation)
        self.assertEqual(new_env.factory.solver_preference_list,
                          env.factory.solver_preference_list)

        self.assertTrue("mathsat-smt" in new_env.factory.all_solvers())
        self.assertTrue("z3-smt" in new_env.factory.all_solvers())


    def test_write_shortcut(self):
        read_configuration(self._get_config_path("config1.ini"))

        tmp = self._get_tmp_file()
        fname = tmp.name

        write_configuration(fname)

        new_env = Environment()
        read_configuration(fname, new_env)
        try:
            os.remove(fname)
        except WindowsError:
            print("Error deleting file...")

        env = get_env()
        self.assertEqual(new_env.enable_infix_notation,
                          env.enable_infix_notation)
        self.assertEqual(new_env.factory.solver_preference_list,
                          env.factory.solver_preference_list)

        self.assertTrue("mathsat-smt" in new_env.factory.all_solvers())
        self.assertTrue("z3-smt" in new_env.factory.all_solvers())

    def test_nonexistent_conf(self):
        with self.assertRaises(PysmtIOError):
            read_configuration("/tmp/nonexistent")

    def test_bad_conf(self):
        with self.assertRaises(Exception):
            read_configuration(self._get_config_path("config_bad.ini"))


if __name__ == '__main__':
    main()
